#!/usr/bin/perl

use warnings;
use strict;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.cpp$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <provider_specializations.cpp>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
open (HEADER, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";

print HEADER "#include \"ComponentModel/Connector.h\"\n\n";

my $index = 0;

PROCESSSHM: foreach my $shm ($cfg->getSharedMemories()) {
	my $shmsym = $shm->shmSymbol();
	my $ctype = $shm->areaType();
	my @writers = @{$shm->getWriters()};
	my $jimpl = 0;

	# If there is a C writer we do not need to create an extra init instance
	foreach my $writer (@writers) {
		my $app = lc($writer->{osapp}->getName());
		$app =~ s/app$//;
		$app =~ s/^[Jj]//; # strip J prefix of Java app name pendant
		next PROCESSSHM if $cfg->hasOSFeature("cfcopter_${app}_c");
		$jimpl = 1 if $cfg->hasOSFeature("cfcopter_${app}_java");
	}
	next unless $jimpl;

	my $shmk = $shm->{name};
	$shmk =~ s/_$ctype//;

	if ( $shmk =~ /^Default_/ ) {
		$shmk = "DefaultProviderID";
	}

	print HEADER "// Writer " . $writers[0]->{osapp}->getName() . "\n";
	print HEADER "static Provider<$ctype, $shmk> _initdummy".$index++.";\n\n";
}
close(HEADER);

