#!/usr/bin/perl

use warnings;
use strict;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.h$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <provider_specializations.h>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
open (HEADER, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";

print HEADER "#ifndef PROVIDER_SPECIALIZATIONS_H\n";
print HEADER "#define PROVIDER_SPECIALIZATIONS_H\n\n";

print HEADER "#include \"sharedmemory_objects.h\"\n";

foreach my $shm ($cfg->getSharedMemories()) {
	my $shmsym = $shm->shmSymbol();
	my $ctype = $shm->areaType();

	my $shmk = $shm->{name};
	$shmk =~ s/_$ctype//;

	if ( $shmk =~ /Default/ ) {
		$shmk = "DefaultProviderID";
	}

	if( $shmk =~ /(.*)_template_(.*)_specialisation_(.*)/ ) {
		my $className = $1;
		print HEADER "template <";
		my @templateParameters = split('_', $2);
		for ( my $i=0; $i <= $#templateParameters; $i+=2 ) {
			print HEADER $templateParameters[$i] . ' ' . $templateParameters[$i+1];
			if ( $i != $#templateParameters-1 ){
				print HEADER ',';
			}
		}
		print HEADER "> class $className;\n";

		print HEADER "typedef $className<";
		my @templateSpecialisations = split('_', $3);
		for ( my $i=0; $i <= $#templateSpecialisations; $i++ ) {
			print HEADER $templateSpecialisations[$i];
			if ( $i != $#templateSpecialisations ){
				print HEADER ',';
			}
		}
		print HEADER '> ';
		
		$shmk = $className;
		for ( my $i=0; $i <= $#templateSpecialisations; $i++ ) {
			$shmk .= '_' . $templateSpecialisations[$i];
		}
		print HEADER "$shmk;\n";
	} else {
		print HEADER "class $shmk;\n";
	}
	print HEADER "template<> class Provider<$ctype, $shmk> : public BasicConnector<$ctype, $shmk> {\n";
	print HEADER "\tpublic:\n";
	print HEADER "\t\tProvider() : BasicConnector<$ctype, $shmk>(&$shmsym) {};\n";
	print HEADER <<eofjs;
    $ctype* operator->()
    {
        return &$shmsym;
    }

    $ctype* const operator &()
    {
        return &$shmsym;
    }

    operator $ctype&()
    {
        return $shmsym;
    }
eofjs
	print HEADER "};\n\n";
}

print HEADER "\n#endif\n";

close(HEADER);

