# Expected Environent:
#
# CIAOBASE:    directory with the CiAO os source (i.e., the trunk/os subdirectory)

CIAOVARIANT := $(CIAOBASE)/kconf/config

# default target 
default : all

# Root directory of CiAO
ROOT_DIR ?= $(CIAOVARIANT)
include $(ROOT_DIR)/make/standard.mk

APP     := $(shell basename $(PWD))

-include Keso_$(APP)/Makefile

# Compile and link the test sources
SRC_DIR := $(APPS_DIR)/$(APP)
EXTRA_AH_DIR := $(SRC_DIR)
OUT_DIR := $(SRC_DIR)

UPPERUSER=$(shell echo $(USER) | tr '[:lower:]' '[:upper:]')  
# Additional g++ Flags
EXTRACXXFLAGS += -I$(PWD) -I$(PWD)/copter/System -I$(PWD)/copter -DBUILDELF -D_OS_CIAO -fdata-sections -ffunction-sections -DUSER_$(UPPERUSER)
CXXFLAGS += -g -O3 -finline $(EXTRACXXFLAGS) -I$(CIAO_DIR) -D_OS_CIAO_DISABLE -DEASYRUN_TC1796 -DUSER_$(UPPERUSER) -include copter/System/Configuration/CopterSoftwareConfig.h
#CXXFLAGS += -g -O3 -finline $(EXTRACXXFLAGS) -I$(CIAO_DIR) -D_OS_CIAO_DISABLE -DEASYRUN_TC1796 -DUSER_SIJESCHE_WIN
AGXXFLAGS := --keep_woven
LDFLAGS := $(CXX_PATH)/../tricore/lib/ldscripts/elf32tricore.x -T $(OUT_DIR)/ldciao.x -M -Map ciao.map --warn-common --allow-multiple-definition
LIBS = os gcc m c

AGXX ?= $(AGXX_PATH)/ag++ --c_compiler $(CXX_PATH)/$(GCC_ARCH)-g++-wrapper $(AGXXFLAGS) $(CIAO_AGXXFLAGS) $(CXXFLAGS)

MODULES	:= architecture generate ciao compile gcc

# include modules
ifneq "$(MODULES)" ""
    include $(patsubst %,$(MODULES_DIR)/%.mk,$(MODULES))
endif

MODULES_CLEAN += app
MODULES_ALL   += app

all_app: $(GEN) $(APP).elf

clean_app:
	@find $(OUT_DIR) -name "*.o" -name "*.acc" -name "$(APP).elf" -delete

# process 'all' targets in every MODULE and generate object files
all: $(patsubst %,all_%,$(MODULES_ALL))  
	
# process 'clean' targets in every MODULE and generate object files
clean: $(patsubst %,clean_%,$(MODULES_CLEAN))
	
.PHONY: all clean all_app clean_app

mrproper:
	@[ -d "$(CIAOVARIANT)" ] || ( echo 'Set $$CIAOVARIANT first' ; exit 1 )
	find "$(CIAOVARIANT)" \( -name '*.acp' -o -name '*.a' -o -name '*.o' -o -name '*.acc' -o -name '*.elf' -o -name ciao.map \) -a -delete

