#include <tc1796b/gpta0.h>
#include "hw/triboard1796.h"
#include "stopwatch.h"

unsigned long long stop_watch_start_value[NUM_CHANNELS];
unsigned long long stop_watch_stop_value[NUM_CHANNELS];

unsigned long long stop_watch_timer_tick_ns;

void STOPWATCH_Init(unsigned long freq) {
  unsigned long mod_freq = 75000000L;
  unsigned long desired_freq = 1;
  unsigned long gpta_divider = 0;
  unsigned long i = 0;

  for(i = 0;desired_freq < freq;i++) {
    desired_freq *= 1000; 
  }

  for(i = 0;i < 14;i++) {
    if((gpta_divider = (mod_freq / (1 << i)) / desired_freq) > 0x3FF) {
      continue;
    }
    break;
  }
	
	stop_watch_timer_tick_ns = 1000000000 / (mod_freq / ((1 << i) * gpta_divider));
	gpta_divider = 0x3FF - gpta_divider;
	
	hw::_tc::unlock_wdtcon();

  /**
   * enable gpta0 OK
   * enable suspend ??
   **/
  GPTA0_CLC.reg = (GPTA0_CLC.reg & ~0x1);
  
  /**
   * normal divider mode
   * set step to the value computed above: gpta_divider
   **/
  GPTA0_FDR.reg &= ~0x20000000;
  GPTA0_FDR.reg = (GPTA0_FDR.reg & ~0x3000F3FF) | ((1 << 14) | gpta_divider);

	hw::_tc::lock_wdtcon();

  /**
   * - no request will generated
   * - do not care about TGE flag source
   * - choose clock bus line CLK2 (== f_gpta)
   **/
	GPTA0_GTCTR0.reg |= 0x20;

  /**
   * - set DFA02 to value computed above: i
   **/
	GPTA0_CKBCTR.reg = (GPTA0_CKBCTR.reg & ~0xF) | i;

  /**
   * - enable clock for gpta0
   * - start time gt0 of gpta0
   **/
	GPTA0_EDCTR.reg |= 0x101;
}

void STOPWATCH_Start(unsigned char channel) {
  stop_watch_start_value[channel] = (unsigned long long) GPTA0_GTTIM0.reg & 0x00FFFFFF;
}

unsigned long long STOPWATCH_Stop(unsigned char channel) {
  stop_watch_stop_value[channel]  = (unsigned long long) GPTA0_GTTIM0.reg & 0x00FFFFFF;
  
  if (stop_watch_stop_value[channel] >= stop_watch_start_value[channel])
  	return stop_watch_stop_value[channel] - stop_watch_start_value[channel];
  else
  	return (0x00FFFFFF - stop_watch_start_value[channel]) + stop_watch_stop_value[channel];
}
