#include <stdlib.h>

#include "common/common.h"
#include "control/command.h"
#include "config.h"

config_elem_t* NewConfigElem() {
	config_elem_t* elem = (config_elem_t*) malloc(sizeof(config_elem_t));
	if(elem == NULL) {
		abort();
	}
	elem->cmd.mtype = MOVE_END;
	elem->cmd.n     = 0;
	elem->next  = NULL;
	elem->prior = NULL;
	
	return elem;
}


config_t* CFG_Create() {
	config_t* conf = (config_t*) malloc(sizeof(config_t));
	config_elem_t* elem = NewConfigElem();
	
	if(conf == NULL) {
		abort();
	}
	conf->currentCoil = 0;
	
	elem->cmd.mtype = MOVE_START;
	conf->firstElem = elem;
	conf->lastElem = elem;
	
	return conf;
}

void CFG_Delete(config_t* config) {
	config_elem_t* toDel;
	
	while(config->firstElem != NULL) {
		toDel = config->firstElem;
		config->firstElem = config->firstElem->next;
		free(toDel);
	}
	
	free(config);
}

void AddConfigElem(config_t* config, config_elem_t* configElem) {
	config->lastElem->next = configElem;
	configElem->prior = config->lastElem;
	config->lastElem = configElem;
	configElem->next  = NULL;
}

void CFG_AddUp(config_t* config, int n) {
	config_elem_t* elem = NewConfigElem();
	elem->cmd.mtype = MOVE_UP;
	elem->cmd.n     = n;
	
	AddConfigElem(config, elem);
}

void CFG_AddDown(config_t* config, int n) {
	config_elem_t* elem = NewConfigElem();
	elem->cmd.mtype = MOVE_DOWN;
	elem->cmd.n     = n;
	
	AddConfigElem(config, elem);
}

void CFG_AddHold(config_t* config, int timeInMS) {
	config_elem_t* elem = NewConfigElem();
	elem->cmd.mtype = MOVE_HOLD;
	elem->cmd.n     = timeInMS;
	
	AddConfigElem(config, elem);
}

void CFG_AddRelease(config_t* config, int timeInMS) {
	config_elem_t* elem = NewConfigElem();
	elem->cmd.mtype = MOVE_RELEASE;
	elem->cmd.n     = timeInMS;
	
	AddConfigElem(config, elem);
}

void CFG_AddEnd(config_t* config) {
	config_elem_t* elem = NewConfigElem();
	elem->cmd.mtype = MOVE_END;
	elem->cmd.n     = 0;
	
	AddConfigElem(config, elem);
}

