#
# Makefile for simple CiAO test programs
#

# Expected Environent:
#
# CIAOBASE:    directory with the CiAO os source (i.e., the trunk/os subdirectory)

CIAOVARIANT := $(CIAOBASE)/kconf/config

# default target 
.PHONY: default
default: all

# Root directory of CiAO
ROOT_DIR ?= $(CIAOVARIANT)
include $(ROOT_DIR)/make/standard.mk

APP      := $(shell basename $(PWD))

-include Keso_$(APP)/Makefile

# Compile and link the test sources
SRC_DIR      := $(PWD)
EXTRA_AH_DIR := $(SRC_DIR)
OUT_DIR      := $(SRC_DIR)

# Additional g++ Flags
#CXXFLAGS += -g -O3 -finline -I$(CIAO_DIR) -fdata-sections -DBUILDELF -D_OS_CIAO -D_REENT_ONLY $(EXTRACXXFLAGS) 
CXXFLAGS += -g -O3 -finline -I$(CIAO_DIR) -fdata-sections -DBUILDELF -D_OS_CIAO $(EXTRACXXFLAGS) 
AGXXFLAGS := --keep_woven
LDFLAGS := $(CXX_PATH)/../tricore/lib/ldscripts/elf32tricore.x -T $(OUT_DIR)/ldciao.x -M -Map ciao.map --warn-common --allow-multiple-definition
LIBS = m c gcc
MODULES	:= architecture generate ciao compile gcc

# include modules
ifneq "$(MODULES)" ""
    include $(patsubst %,$(MODULES_DIR)/%.mk,$(MODULES))
endif

MODULES_CLEAN += app
MODULES_ALL   += app

all_app: $(GEN) $(APP).elf

clean_app:
	@find $(OUT_DIR) -name "*.o" -name "*.acc" -name "$(APP).elf" -delete

# process 'all' targets in every MODULE and generate object files
all: $(patsubst %,all_%,$(MODULES_ALL))  
	
# process 'clean' targets in every MODULE and generate object files
clean: $(patsubst %,clean_%,$(MODULES_CLEAN))
	
.PHONY: all clean all_app clean_app

mrproper:
	@[ -d "$(CIAOVARIANT)" ] || ( echo 'Set $$CIAOVARIANT first' ; exit 1 )
	find "$(CIAOVARIANT)" \( -name '*.acp' -o -name '*.a' -o -name '*.o' -o -name '*.acc' -o -name '*.elf' -o -name ciao.map \) -a -delete

