#
# MODULE: gcc
#
# setup gcc and binutils according to the current architecture


# common flags
CXXFLAGS 	+= -Wall -fno-rtti -fno-exceptions -ffunction-sections -fno-builtin-printf
CXXFLAGS 	+= -I $(SRC_DIR)
LDFLAGS 	+= -static
#AGXXFLAGS 	+= -p $(SRC_DIR) --real-instances --no_warn_macro 
#AGXXFLAGS   += -r $(CIAO_DIR)/ciao.acp --real-instances --no_warn_macro --keep_woven --no_line

## We need --real-instances since CiAO uses some types for which the appropriate size is computed by a template
## If --real-instances is missing, pointcut expressions on functions where such a type occurs in the function's
## signature will not match
AGXXFLAGS += --real-instances --no_warn_macro --keep_woven --no_line -k

export ACNEWREPO:=$(CIAO_DIR)/ciao.acp
#the following option apparently causes the weaver to ignore the (missing) -DBUILDELF in the commandline
#--config $(CIAO_DIR)/puma.config

# merge -p dirs into AGXXFLAGS (one for CiAO and, optionally, one for the app)
AGXXFLAGS	+= $(foreach p_dir,$(CIAO_DIR) $(EXTRA_AH_DIR),-p $(p_dir))


# compilation-unit specific flags (may override common flags)
# 
# compilation-unit specific CXXFLAGS can be defined by adding a magic
# C++ comment line to the translation unit:
#
# // CXXFLAGS_EXTRA=-foption1 -foption2
#
# - this line has to start with the "//" comment specifier
# - everything after the "CXXFLAGS_EXTRA=" is added to the compilers command line
# - only the first 10 lines of each compilation unit are searched for a CXXFLAGS_EXTRA comment
# - only translation units are searched, hence it is not possible to add extra flags by an included file

CXXFLAGS_EXTRA = $(shell sed -n -e '1,10s/^\/\/.*CXXFLAGS_EXTRA=//p' '$<' )


# include files with definitions for current architecture
include $(MODULES_DIR)/gcc/$(ARCH).mk

# default path to compilers
# only set if not already set by environment or arch-specific makefile
CXX_PATH	?= /proj/i4ciao/tools/gcc/bin
AGXX_PATH	?= /proj/i4ciao/tools/bin
ACXX_PATH	?= /proj/i4ciao/tools/bin

# standard compiler
CXX ?= $(CXX_PATH)/$(GCC_ARCH)-g++
CXX += $(CXXFLAGS_ARCH) $(CXXFLAGS) $(CIAO_CXXFLAGS) $(CXXFLAGS_EXTRA)

AS	?= $(CXX_PATH)/$(GCC_ARCH)-g++
AS	+= $(ASFLAGS_ARCH) $(ASFLAGS)

# set weaver for this g++ version
AGXX ?= $(AGXX_PATH)/ag++ --c_compiler $(CXX_PATH)/$(GCC_ARCH)-g++ $(AGXXFLAGS) $(CIAO_AGXXFLAGS) $(CXXFLAGS)
AGXX += --Xcompiler $(CXXFLAGS_ARCH) $(CIAO_CXXFLAGS) $(CXXFLAGS_EXTRA)
 
# set actual compiler to use
XX ?= AGXX

# don't generate code for initializing bss segement
# -fno-zero-initialized-in-bss
	 

