// $Id: MemPort.h 995 2009-08-11 14:26:31Z mike $

#ifndef __MemPort_h__
#define __MemPort_h__

/**
 * \file MemPort.h
 * \brief memory mapped i/o port for AVR
 */
 
/*! \class MemPort
 * \brief memory mapped i/o port
 *
 * This class implements a memory mapped i/o port on the AVR architecture.
 * As ports can be of 8/16/32... bits width, the class is defined as a
 * template.
 */

namespace ciao {
    
    template <typename T, IntPtr PORT = 0> class MemPort {

    public:
	//! read from the port
	operator T () volatile { return *(volatile T*)PORT; }
	
	//! write to the port
	MemPort& operator = (T val) { *(volatile T*)PORT = val; return *this; }

	//! clear bits with boolean 'and' operation
	MemPort& operator &= (T val) {
	    *(volatile T*)PORT &= val; 
	    return *this;
	}

	//! set bits with boolean 'or' operation
	MemPort& operator |= (T val) {
	    *(volatile T*)PORT |= val;
	    return *this; 
	}
    };

    template <typename T> class MemPort<T, 0> {

	//! pointer to the port in memory
	volatile T * const _port;

    public:
	//! construct a memory mapped port object
	MemPort (IntPtr p) : _port ((volatile T*)p) {}

	//! read from the port
	operator T () volatile { return *_port; }
	
	//! write to the port
	MemPort& operator = (T val) { *_port = val; return *this; }

	//! clear bits with boolean 'and' operation
	MemPort& operator &= (T val) { *_port &= val; return *this; }

	//! set bits with boolean 'or' operation
	MemPort& operator |= (T val) { *_port |= val; return *this; }
    };

} // namespace ciao

#endif // __MemPort_h__
