/*
 * $Id: vectab.x 682 2008-02-14 15:48:19Z sirupfis $
 * created:   08.03.2006 lohmann
 */

/**
 * \file vectab.x
 * \brief defines linker sections for the CiAO IRQ and Trap vector tables
 * 
 * On Tricore each "vector table entry" has a size of 2^5==32 bytes. The PC 
 * address is calculatd by: PC = BIV | (PIPN << 5) 
 * As PIPN is effective 8 bit (256 levels), the base address (BIV) has to be
 * aligned to a 2^(5+8) == 8192 (0x2000) byte boundary.
 * 
 * The same mechanism is used for traps. However, as Tricore defines only
 * 8 trap classes, the trap class number is effecive 3 bit, hence the BTV
 * has to be aligend to a 2^(5+3) == 256 (0xFF) boundary
 *
 */
 
SECTIONS {

	/* to ensure that the startup_code is *always* placed at the beginning */

	.startup : {
		KEEP (*(.startup_code))
		. = ALIGN(8);
	} > ext_cram = 0

	/* we place the irqtabs in the internal code ram, which may be changed
	  if this for some reason is not suitable. The actual IRQ handlers should,
	  however, be placed in the same memory region, to be able to jump
	  PC-relative to them
    */	

    	/* quickfix: irqtabs now in external code ram to allow PC-relative jumps
		     to further interrupt management*/
	
	.ciao_irqtab : {
		. = ALIGN(0x2000);
		KEEP (*(.ciao_irqtab))
		KEEP (*(.ciao_irqtab.handler))
	} > ext_cram
	
	.ciao_traptab : {
		. = ALIGN(0x00ff);
		KEEP (*(.ciao_traptab))
		KEEP (*(.ciao_traptab.handler))
	} > ext_cram      
	
	.pcptext : {
		PCODE_BASE = . ;
		KEEP (*(.pcptext))
		KEEP (*(.pcptext.*))
		. = ALIGN(8) ;
		PCODE_END = . ;
	} > pcp_text AT> ext_cram
}
