;
;  Startup-Code für x86_64
;

STACKSIZE:  equ 65536
CGA:        equ 0xB8000

[GLOBAL pml4]
pml4:       equ 0x100000

; Ab hier werden die Pagetable-Einträge abgespeichert
[GLOBAL pagetable_start]
pagetable_start:  equ 0x103000

[GLOBAL pagetable_end]
pagetable_end:  equ 0x200000

[GLOBAL _start]
[GLOBAL idt]

[EXTERN __main]
[EXTERN main]
[EXTERN guardian]

[SECTION .text.startup]

	jmp	_start		; Sprung an den Start (für den konventionellen System-Code)
	align	4		; Auf Alignment achten (GRUB braucht das)

;
;  Konstanten für Multiboot Header
;
;FLAGS:	equ		(1 << 0) | (1 << 1) | (1 << 2) ; (Speicher, Bootdevice, Kernel-Cmdline)
FLAGS:	equ		(1 << 0) | (1 << 1)            ; (Speicher, Bootdevice)

;
;  Multiboot Header (für GRUB)
;
        dd      0x1BADB002         ; Multiboot Magic
        dd      FLAGS              ; Flags
        dd      0xE4524FFE - FLAGS ; Checksum
        dd      0                  ; header_addr (wird ignoriert)
        dd      0                  ; load_addr (wird ignoriert)
        dd      0                  ; load_end_addr (wird ignoriert)
        dd      0                  ; bss_end_addr (wird ignoriert)
        dd      0                  ; entry_addr (wird ignoriert)
        dd      0                  ; mode_type (wird ignoriert)
        dd      0                  ; width (wird ignoriert)
        dd      0                  ; height (wird ignoriert)
        dd      0                  ; depth (wird ignoriert)

;
;  GRUB Einsprungspunkt
;
_start:
[BITS 32]

; GCC-kompilierter Code erwartet das so.

	cld

; Interrupts ausschalten
  cli

; Neue Segmentdeskriptoren setzen
  lgdt    [gdt_80]

; Globales Datensegment

	mov	  eax, 3 * 0x8
	mov	  ds, ax
	mov	  es, ax
	mov	  fs, ax
	mov	  gs, ax

; Stack festlegen

	mov	  ss, ax
	mov   esp, init_stack+STACKSIZE

; Multiboot Datenstrukturen
	mov	eax, [ebx+0]	            ; Flags auslesen
	mov	[MULTIBOOT_FLAGS], eax

	mov	eax, [ebx+4]                ; Größe des konventionellen Speichers auslesen
	mov	[MULTIBOOT_LOWER_MEM], eax

	mov	eax, [ebx+8]                ; Größe des erweiterten Speichers auslesen
	mov	[MULTIBOOT_UPPER_MEM], eax

	mov	eax, [ebx+12]               ; Bootdevice auslesen
	mov	[MULTIBOOT_BOOTDEVICE], eax

	mov	eax, [ebx+16]               ; Kernel-Kommandozeile auslesen (TODO: String kopieren)
	mov	[MULTIBOOT_CMDLINE], eax
  xor eax, eax 
  mov [MULTIBOOT_CMDLINE+4], eax

	mov     eax, [ebx+20]               ; Anzahl Multiboot-Module auslesen
	mov     [MULTIBOOT_MODULES_COUNT], eax

	mov     eax, [ebx+24]               ; Multiboot-Modul-Startadresse
	mov     [MULTIBOOT_MODULES_ADDRESS], eax
  xor     eax, eax 
  mov     [MULTIBOOT_MODULES_ADDRESS+4], eax 

; Umschalten in den 64-Bit Long-Mode
  jmp init_longmode

longmode_start:
[BITS 64]

  ; Text im 64-Bit-Modus ausgeben
;    mov     rax, 'B l u b ' 
;    mov     [CGA],rax 
; bla: jmp bla
  

; Unterbrechungsbehandlung sicherstellen

	call	setup_idt
	call	reprogram_pics

; Aufruf des C-Codes

	call  __main          ; Aufruf der init()-Funktionen
	call	main            ; C/C++ Level System
  cli 
	hlt


;
;  Initialisierung des 64-Bit Long Modes
;
init_longmode:
[BITS 32]
        ; Adresserweiterung aktivieren (PAE)
        mov     eax, cr4 
        or      eax, 1 << 5 
        mov     cr4, eax

        call setup_paging
 
        ; Long-Mode im Compatibility-Mode aktivieren
        mov     ecx, 0x0C0000080        ; EFER MSR 
        rdmsr 
        or      eax, 1 << 8
        wrmsr 

        ; Paging aktivieren
        mov     eax, cr0 
        or      eax, (1 << 31)
        mov     cr0, eax

        ; Sprung ins 64-Bit Codesegment und damit
        ; Aktivierung des 64-Bit Long-Modes
        jmp     1 * 0x8:longmode_start


setup_paging:
        ; Seitentabelle ausnullen
        mov     edi, 0x100000
        mov     ecx, 0x8000
        mov     eax, 0
        rep     stosd
 
        ; (provisorische) Seitentabelle mit 2 MB Seitengröße anlegen,
        ; die die ersten 8 MB direkt auf den
        ; physikalischen Speicher abbildet.
        ; Dies ist notwendig, da eine funktionierende Seitentabelle für den Long-Mode
        ; vorausgesetzt wird.
        ; Optional legt CiAO bei der Initialisierung eine eigene Seitentabelle an,
        ; die zur Laufzeit manipuliert werden kann.
        ; Bis dahin darf der Bereich zwischen 0x100000 und 0x200000 NICHT benutzt werden,
        ; da dort die provisorische Seitentabelle abgelegt wird.

        ; PML4 (1. Stufe)
        mov     dword [pml4+0], 0x101000 | 0x7
        mov     dword [pml4+4], 0

        ; PDPE (2. Stufe)
        mov     dword [0x101000], 0x102000 | 0x7
        mov     dword [0x101004], 0

        ; PDE (3. Stufe)
        mov     dword [0x102000], 0x000000 | 0x87  ; 1. Seite (0x000000 - 0x1FFFFF)
        mov     dword [0x102004], 0x0

        mov     dword [0x102008], 0x200000 | 0x87  ; 2. Seite (0x200000 - 0x3FFFFF)
        mov     dword [0x10200C], 0x0
        
        mov     dword [0x102010], 0x400000 | 0x87  ; 3. Seite (0x400000 - 0x5FFFFF)
        mov     dword [0x102014], 0x0
        
        mov     dword [0x102018], 0x600000 | 0x87  ; 4. Seite (0x600000 - 0x7FFFFF)
        mov     dword [0x10201C], 0x0

        ; Basiszeiger auf PML4 setzen
        mov     eax, 0x100000
        mov     cr3, eax
        ret

;
;  Reset-Funktion
;
reset:  mov eax, 0x472
        mov (eax), 0x1234

reset2:
        mov     edx, 0x64
        mov     eax, 0xfe
        out     dx, ax
        jmp reset2

; Default Interrupt Behandlung
[BITS 64]
; Spezifischer Kopf der Unterbrechungsbehandlungsroutinen

%macro wrapper 1
wrapper_%1:
  push rbp
  mov rbp, rsp
	push	rax
	mov	al,%1 
	jmp	wrapper_body
%endmacro

; ... wird automatisch erzeugt.
%assign i 0
%rep 256
wrapper i
%assign i i+1
%endrep

; Gemeinsamer Rumpf
wrapper_body:
	cld				; das erwartet der gcc so.
  push  rcx		; Sichern der fluechtigen Register
  push  rdx
  push  rdi
  push  rsi
  push  r8
  push  r9
  push  r10
  push  r11

	and	rax,0xff	; Der generierte Wrapper liefert nur 8 Bits
 
	mov   rdi, rax		; Nummer der Unterbrechung uebergeben
	call  guardian
 
  pop	r11			; fluechtige Register wieder herstellen
  pop	r10
  pop r9
  pop r8
  pop rsi
  pop rdi
  pop rdx
  pop rcx

  pop	rax
  pop rbp
	iretq			; fertig!

;
; setup_idt
;
; Relokation der Eintraege in der IDT und Setzen des IDTR

setup_idt:


  mov	rax,wrapper_0	;  ax: Bits 0-15
  mov	rbx,rax
  mov rdx,rax
  shr rdx,32        ; edx: Bits 32-63
  shr	rbx,16			  ;  bx: Bits 16-31

  mov r10, idt      ; Zeiger auf aktuellen Interrupt-Gate
  mov	rcx,255			  ; Zaehler
.loop:	add	[r10+0],ax
        adc	[r10+6],bx
        adc [r10+8],edx
        add r10, 16       ; Zeiger inkrementieren
        dec	rcx
        jge	.loop

	lidt	[idt_descr]
	ret

;
; reprogram_pics
;
; Neuprogrammierung der PICs (Programmierbare Interrupt-Controller), damit
; alle 15 Hardware-Interrupts nacheinander in der idt liegen.

reprogram_pics:
	mov	al,0x11   ; ICW1: 8086 Modus mit ICW4
	out	0x20,al
	call	delay
	out	0xa0,al
	call	delay
	mov	al,0x20   ; ICW2 Master: IRQ # Offset (32)
	out	0x21,al
	call	delay
	mov	al,0x28   ; ICW2 Slave: IRQ # Offset (40)
	out	0xa1,al
	call	delay
	mov	al,0x04   ; ICW3 Master: Slaves an IRQs
	out	0x21,al
	call	delay
	mov	al,0x02   ; ICW3 Slave: Verbunden mit IRQ2 des Masters
	out	0xa1,al
	call	delay
	mov	al,0x03   ; ICW4: 8086 Modus und automatischer EIO
	out	0x21,al
	call	delay
	out	0xa1,al
	call	delay

	mov	al,0xff		; Hardware-Interrupts durch PICs
	out	0xa1,al		; ausmaskieren. Nur der Interrupt 2,
	call	delay		; der der Kaskadierung der beiden
	mov	al,0xfb		; PICs dient, ist erlaubt.
	out	0x21,al

	ret

; delay
;
; Kurze Verzoegerung fuer in/out Befehle.

delay:
	jmp	.L2
.L2:	ret

[SECTION .data]

;
;  Segment-Deskriptoren
;
gdt:
  dw  0,0,0,0   ; NULL Deskriptor

  ; 64-Bit-Codesegment-Desktriptor
  dw  0xFFFF    ; 4Gb - (0x100000*0x1000 = 4Gb)
  dw  0x0000    ; base address=0
  dw  0x9A00    ; code read/exec
  dw  0x00AF    ; granularity=4096, 386 (+5th nibble of limit), Long-Mode
  
  ; 32-Bit-Codesegment-Desktriptor
  dw  0xFFFF    ; 4Gb - (0x100000*0x1000 = 4Gb)
  dw  0x0000    ; base address=0
  dw  0x9A00    ; code read/exec
  dw  0x00CF    ; granularity=4096, 386 (+5th nibble of limit)

  ; Datensegment-Desktriptor
  dw  0xFFFF    ; 4Gb - (0x100000*0x1000 = 4Gb)
  dw  0x0000    ; base address=0
  dw  0x9200    ; data read/write
  dw  0x00CF    ; granularity=4096, 386 (+5th nibble of limit)

gdt_80:
  dw  4*8-1     ; GDT Limit=24, 4 GDT Eintraege - 1
  dq  gdt       ; Adresse der GDT


;  'interrupt descriptor table' mit 256 Eintraegen.

idt:

%macro idt_entry 1
	dw	(wrapper_%1 - wrapper_0) & 0xffff
	dw	0x0000 | 0x8 * 1
	dw	0x8e00
	dw	((wrapper_%1 - wrapper_0) & 0xffff0000) >> 16
  dq  ((wrapper_%1 - wrapper_0) & 0xffffffff00000000) >> 32
%endmacro

; ... wird automatisch erzeugt.

%assign i 0
%rep 256
idt_entry i
%assign i i+1
%endrep

idt_descr:
	dw	256*8-1		; idt enthaelt 256 Eintraege
	dq	idt

;   Stack und interrupt descriptor table im BSS Bereich

[SECTION .bss]

[GLOBAL MULTIBOOT_FLAGS]
[GLOBAL MULTIBOOT_LOWER_MEM]
[GLOBAL MULTIBOOT_UPPER_MEM]
[GLOBAL MULTIBOOT_BOOTDEVICE]
[GLOBAL MULTIBOOT_CMDLINE]
[GLOBAL MULTIBOOT_MODULES_COUNT]
[GLOBAL MULTIBOOT_MODULES_ADDRESS]

MULTIBOOT_FLAGS:			resd	1
MULTIBOOT_LOWER_MEM:		resd	1
MULTIBOOT_UPPER_MEM:		resd	1
MULTIBOOT_BOOTDEVICE:		resd	1
MULTIBOOT_CMDLINE:			resq	1
MULTIBOOT_MODULES_COUNT:	resd	1
MULTIBOOT_MODULES_ADDRESS:	resq	1

global init_stack:data (init_stack.end - init_stack)
init_stack:
	;resb	4096
	;resb	33000
	resb   STACKSIZE
	;resb	500000
.end:
