;
;   System
;

[GLOBAL _start]
[GLOBAL idt]

[EXTERN __main]
[EXTERN main]
[EXTERN guardian]

init_stack.size:  equ   0x10000

[SECTION .text.startup]

	jmp	_start		; Sprung an den Start (für den konventionellen System-Code)
	align	4		; Auf Alignment achten (GRUB braucht das)

;
;  Konstanten für Multiboot Header
;
;FLAGS:	equ		(1 << 0) | (1 << 1) | (1 << 2) ; (Speicher, Bootdevice, Kernel-Cmdline)
FLAGS:	equ		(1 << 0) | (1 << 1)            ; (Speicher, Bootdevice)

;
;  Multiboot Header (für GRUB)
;
        dd      0x1BADB002         ; Multiboot Magic
        dd      FLAGS              ; Flags
        dd      0xE4524FFE - FLAGS ; Checksum
        dd      0                  ; header_addr (wird ignoriert)
        dd      0                  ; load_addr (wird ignoriert)
        dd      0                  ; load_end_addr (wird ignoriert)
        dd      0                  ; bss_end_addr (wird ignoriert)
        dd      0                  ; entry_addr (wird ignoriert)
        dd      0                  ; mode_type (wird ignoriert)
        dd      0                  ; width (wird ignoriert)
        dd      0                  ; height (wird ignoriert)
        dd      0                  ; depth (wird ignoriert)

_start:

; GCC-kompilierter Code erwartet das so.

	cld

; Globales Datensegment

	;mov	ax,0x10
	;mov	ds,ax
	;mov	es,ax
	;mov	fs,ax
	;mov	gs,ax

; Stack festlegen

	;mov	ss,ax
	;mov	esp,init_stack+4096
	;mov	esp,init_stack+65536
	;mov	esp,init_stack+131072
	;mov	esp,init_stack+60000

	mov esp, init_stack.end 

; Multiboot Datenstrukturen
	mov	eax, [ebx+0]	            ; Flags auslesen
	mov	[MULTIBOOT_FLAGS], eax

	mov	eax, [ebx+4]                ; Größe des konventionellen Speichers auslesen
	mov	[MULTIBOOT_LOWER_MEM], eax

	mov	eax, [ebx+8]                ; Größe des erweiterten Speichers auslesen
	mov	[MULTIBOOT_UPPER_MEM], eax

	mov	eax, [ebx+12]               ; Bootdevice auslesen
	mov	[MULTIBOOT_BOOTDEVICE], eax

	mov	eax, [ebx+16]               ; Kernel-Kommandozeile auslesen (TODO: String kopieren)
	mov	[MULTIBOOT_CMDLINE], eax

	mov     eax, [ebx+20]               ; Anzahl Multiboot-Module auslesen
	mov     [MULTIBOOT_MODULES_COUNT], eax

	mov     eax, [ebx+24]               ; Multiboot-Modul-Startadresse
	mov     [MULTIBOOT_MODULES_ADDRESS], eax



; Unterbrechungsbehandlung sicherstellen

	call	setup_idt
	call	reprogram_pics

; Aufruf des C-Codes

	call    __main          ; Aufruf der init()-Funktionen
	call	main            ; C/C++ Level System
	hlt

; Default Interrupt Behandlung

; Spezifischer Kopf der Unterbrechungsbehandlungsroutinen

%macro wrapper 1
wrapper_%1:
	push	eax
	mov	al,%1
	jmp	wrapper_body
%endmacro

; ... wird automatisch erzeugt.
%assign i 0
%rep 256
wrapper i
%assign i i+1
%endrep

; Gemeinsamer Rumpf
wrapper_body:
	cld				; das erwartet der gcc so.
	push	ecx		; Sichern der fluechtigen Register
	push	edx
	and	eax,0xff	; Der generierte Wrapper liefert nur 8 Bits
	push	eax		; Nummer der Unterbrechung uebergeben
	call	guardian
	add	esp,4		; Parameter vom Stack entfernen
	pop	edx			; fluechtige Register wieder herstellen
	pop	ecx
	pop	eax
	iret			; fertig!

;
; setup_idt
;
; Relokation der Eintraege in der IDT und Setzen des IDTR

setup_idt:
	mov	eax,wrapper_0	; ax: niederwertige 16 Bit
	mov	ebx,eax
	shr	ebx,16			; bx: hoeherwertige 16 Bit
	mov	ecx,255			; Zaehler
.loop:	add	[idt+8*ecx+0],ax
	adc	[idt+8*ecx+6],bx

	;; fix for grub 1.98
	;; set correct segment-selector for 32-bit code-segment in idt
	push	eax
	mov	ax, cs
	add	[idt+8*ecx+2],ax
	pop	eax
	
	dec	ecx
	jge	.loop

	lidt	[idt_descr]
	ret

;
; reprogram_pics
;
; Neuprogrammierung der PICs (Programmierbare Interrupt-Controller), damit
; alle 15 Hardware-Interrupts nacheinander in der idt liegen.

reprogram_pics:
	mov	al,0x11   ; ICW1: 8086 Modus mit ICW4
	out	0x20,al
	call	delay
	out	0xa0,al
	call	delay
	mov	al,0x20   ; ICW2 Master: IRQ # Offset (32)
	out	0x21,al
	call	delay
	mov	al,0x28   ; ICW2 Slave: IRQ # Offset (40)
	out	0xa1,al
	call	delay
	mov	al,0x04   ; ICW3 Master: Slaves an IRQs
	out	0x21,al
	call	delay
	mov	al,0x02   ; ICW3 Slave: Verbunden mit IRQ2 des Masters
	out	0xa1,al
	call	delay
	mov	al,0x03   ; ICW4: 8086 Modus und automatischer EIO
	out	0x21,al
	call	delay
	out	0xa1,al
	call	delay

	mov	al,0xff		; Hardware-Interrupts durch PICs
	out	0xa1,al		; ausmaskieren. Nur der Interrupt 2,
	call	delay		; der der Kaskadierung der beiden
	mov	al,0xfb		; PICs dient, ist erlaubt.
	out	0x21,al

	ret

; delay
;
; Kurze Verzoegerung fuer in/out Befehle.

delay:
	jmp	.L2
.L2:	ret

[SECTION .data]

;  'interrupt descriptor table' mit 256 Eintraegen.

idt:

%macro idt_entry 1
	dw	(wrapper_%1 - wrapper_0) & 0xffff
	dw	0x0000		; Segment-Selektor des Code-Segments für Unterbrechungsbehandlungsroutinen
	dw	0x8e00
	dw	((wrapper_%1 - wrapper_0) & 0xffff0000) >> 16
%endmacro

; ... wird automatisch erzeugt.

%assign i 0
%rep 256
idt_entry i
%assign i i+1
%endrep

idt_descr:
	dw	256*8-1		; idt enthaelt 256 Eintraege
	dd	idt

;   Stack und interrupt descriptor table im BSS Bereich

[SECTION .bss]

[GLOBAL MULTIBOOT_FLAGS]
[GLOBAL MULTIBOOT_LOWER_MEM]
[GLOBAL MULTIBOOT_UPPER_MEM]
[GLOBAL MULTIBOOT_BOOTDEVICE]
[GLOBAL MULTIBOOT_CMDLINE]
[GLOBAL MULTIBOOT_MODULES_COUNT]
[GLOBAL MULTIBOOT_MODULES_ADDRESS]

MULTIBOOT_FLAGS:			resd	1
MULTIBOOT_LOWER_MEM:		resd	1
MULTIBOOT_UPPER_MEM:		resd	1
MULTIBOOT_BOOTDEVICE:		resd	1
MULTIBOOT_CMDLINE:			resd	1
MULTIBOOT_MODULES_COUNT:	resd	1
MULTIBOOT_MODULES_ADDRESS:	resd	1

global init_stack:data (init_stack.end - init_stack)
init_stack:
	;resb	4096
	;resb	33000
	;resb 65536
	;resb	500000
	resb  init_stack.size
.end:
