// $Id: I2CProt.h 995 2009-08-11 14:26:31Z mike $
#ifndef __hw_dev_I2CProt_h__
#define __hw_dev_I2CProt_h__

/**
 * \file I2CProt.h
 * \brief Hardware independant part of I2C protocol implementation
 */

#include "util/types.h"

namespace hw { namespace dev {
	
/**
 * \class I2CProt
 * \brief Hardware independant part of I2C protocol implementation
 *
 * This class implements the communication protocol of the I2C bus.
 */
    
template< class I2CChannel > class I2CProt : private I2CChannel {
public:
	void start ();
	void stop();

	void  send (const UInt8&);
	UInt8 recv ();
	bool  ack ();
	void  ack (bool);

	bool start(const UInt8&);
	bool write(const UInt8&);
	bool read(UInt8&, bool);
	bool write(unsigned char*, UInt8);
	bool read(unsigned char*, UInt8);
};



/*     ____
 *         \
 * SCL      \___________
 *
 *     __________
 *               \
 * SDA            \_____
 */
template< class I2CChannel > 
void I2CProt<I2CChannel>::start () {
        sda_hi();
        scl_hi();
        delay();
        sda_lo();
        scl_lo();
}




/*          ____________
 *         /
 * scl ___/
 *
 *                ______
 *               /
 * sda _________/
 */
template< class I2CChannel > 
void I2CProt< I2CChannel >::stop () {
        sda_lo();
        delay();
        scl_hi();
        sda_hi();
}
template< class I2CChannel > 
void I2CProt< I2CChannel >::send (const UInt8& val) {
        UInt8 mask(0x80);

        while( mask != 0 ) {
                (val & mask)==0 ? sda_lo() : sda_hi();
                delay();
                scl_hi();
                delay();
                scl_lo();
                mask >>= 1;
        }
}

template< class I2CChannel > 
UInt8 I2CProt< I2CChannel >::recv () {
        UInt8 res(0);
        UInt8 mask(0x80);

        sda_hi();
        while( mask != 0 ) {
                delay();
                scl_hi();
                if( get_sda()==true )
                        res |= mask;
                delay();
                scl_lo();
                mask >>= 1;
        }
        return res;
}

template< class I2CChannel > 
bool I2CProt< I2CChannel >::ack () {
        sda_hi();
        delay();
        scl_hi();
        bool res = !get_sda();
        delay();
        scl_lo();
        return res;
}

template< class I2CChannel > 
void I2CProt< I2CChannel >::ack (bool val) {
        val ? sda_lo() : sda_hi();
        delay();
        scl_hi();
        delay();
        scl_lo();
}

template< class I2CChannel > 
bool I2CProt< I2CChannel >::write(const UInt8& data) {
        send(data);
        return ack();
}

template< class I2CChannel > 
bool I2CProt< I2CChannel >::read(UInt8& data, bool a) /* pre: scl is low */
{
        data = recv();
        ack(a);
        return true;
}

template< class I2CChannel > 
bool I2CProt< I2CChannel >::write(unsigned char* data, UInt8 len)
{
        while (len--) {
                if ( write(*data++)==false )
                        return false;
        }
        return true;
}

template< class I2CChannel > 
bool I2CProt< I2CChannel >::read(unsigned char* data, UInt8 len)
{
	while (len--) {
		*data = recv();
		ack( len>0 );
		data++;
	}
	return true;
}

template< class I2CChannel > 
bool I2CProt< I2CChannel >::start (const UInt8& adr) {
	start();
	send(adr);
	return ack();
}


    
} } // namespace hw::dev

#endif // __hw_dev_I2CProt_h__
