// $Id: STM.cpp 1315 2010-11-26 20:29:58Z mike $
// Created:   22.05.2006 lohmann

/**
 * \file STM.cpp
 * \brief Implementation of the TC1796 system time unit
 *
 */


#include "STM.h"
#include "hw/triboard1796.h"
#include "ciaoConfig.h"

namespace hw { namespace dev {

//
STM STM::theSTM_;

void STM::init() {

	// set system timer clock control (ENDINIT protected)
/*	Setting SPEN prevents correct alarm operation, it shall only be set by debugging script if really necessary
	STM_CLC.reg =
		SYS_DIV << STM_CLC_RMC_SHIFT 	|	// system clock divider (0-7)
		1		<< STM_CLC_SPEN_SHIFT	|	// OCDS module suspend enable (timer stops if breakpoint is hit)
		1		<< STM_CLC_SBWE_SHIFT	;	// module suspend bit write enable
*/
	STM_CLC.bits.RMC = SYS_DIV;

	// clear pending internal IRQ requests resulting from system reset
	ISRR().bits.CMP0IRR = 0;
	ISRR().bits.CMP1IRR = 0;

	// assign compare-and-match register 0 to SRN0 and 1 to SRN1
	ICR().bits.CMP0OS = 0;
	ICR().bits.CMP1OS = 1;

	// note: Internal IRQs are not enabled by default

	// init SRN priority numbers
	IRQ_SRC0::init();
	IRQ_SRC1::init();

}


UInt32 STM::periodToCompareValues( UInt64 period, UInt32& cmp, unsigned& size, unsigned& start ) {

	unsigned hw = upperHalf( period );
	unsigned lw = lowerHalf( period );
	unsigned lz = _clz( hw );
	unsigned error;

	if( lz == 32 ) {
		cmp  = lw;
		size = 32 - _clz( lw );
		start = 0;
		error = 0;
	}
	else {
		cmp = hw << lz;
		cmp |= ( lw >> (32 - lz ) );
		start = 32 - lz;
		size = 32;
		error = lw & ( 1 << ((32 - lz ) - 1));
	}
	return error;
}

}} // namespace hw::dev
