#ifndef __HW_DEV_CGA_H__
#define __HW_DEV_CGA_H__

#include "IOPort.h"

namespace hw {
namespace dev {

class CGA {
public:
	static const int width = 80;
	static const int height = 25;

private:
	static char* const video; // VRAM

	static const int bytesPerChar = 2;
	static const int bytesPerLine = width * bytesPerChar;

	int cursor_x_pos;
	int cursor_y_pos;

	IOPort port_index;
	IOPort port_data;

	static CGA _theCGA;

public:
	void init() {
		port_index.setAddress(0x3d4);
		port_data.setAddress(0x3d5);
	}

	static CGA& Inst() {
		return _theCGA;
	}

private:
	void updateCursorPos() {
		int pos = width * cursor_y_pos + cursor_x_pos;
		port_index.outb(14);
		port_data.outb(pos >> 8);
		port_index.outb(15);
		port_data.outb(pos);
	}

public:
	// Zugriff auf Curserposition
	void setpos(int x, int y) {
		cursor_x_pos = x;
		cursor_y_pos = y;
		updateCursorPos();
	}
	void getpos(int& x, int& y) const {
		x = cursor_x_pos;
		y = cursor_y_pos;
	}

public:
	// Zeigt ein einzelnes Zeichen c an der Position (x,y) mit Attribut attrib an
	void show(int x, int y, char c, unsigned char attrib = 7) {
		int pos = (y * width + x) * bytesPerChar;
		video[pos] = c;
		video[pos + 1] = attrib;
	}

public:
	// Überschreibt Zeile y mit Leerzeichen und Attribut attrib
	void clearLine(int y, unsigned char attrib = 0) {
		for (int x = 0; x < width; ++x) {
			video[y * bytesPerLine + bytesPerChar * x] = ' ';
			video[y * bytesPerLine + bytesPerChar * x + 1] = attrib;
		}
	}
	// Überschreibt alle Zeilen mit Leerzeichen und Attribut attrib und setzt die Curserposition auf den Anfang
	void clear(unsigned char attrib = 0) {
		for (int y = 0; y < height; ++y)
			clearLine(y, attrib);
		setpos(0, 0);
	}

public:
	// gibt den Text text der Länge length mit Attribut attrib an der aktuellen Curserposition aus
	// Verschiebt den Curser und scrollt gegebenfalls
	void print(const char* text, int length, unsigned char attrib = 7) {
		const char* textEnd = text + length;
		char* vramIter = video + (cursor_y_pos * width + cursor_x_pos) * bytesPerChar;
		for (const char* textIter = text; textIter < textEnd; ++textIter) {
			if (*textIter == '\n') {
				vramIter += (width - cursor_x_pos) * bytesPerChar;
				cursor_x_pos = 0;
				++cursor_y_pos;
				if (cursor_y_pos == height) {
					for (int i = 0; i < (height - 1) * bytesPerLine; ++i) {
						video[i] = video[bytesPerLine + i];
					}
					clearLine(height - 1, 0x0f);
					vramIter -= bytesPerLine;
					--cursor_y_pos;
				}
			} else {
				*vramIter++ = *textIter;
				*vramIter++ = attrib;
				++cursor_x_pos;
				if (cursor_x_pos == width) {
					cursor_x_pos = 0;
					++cursor_y_pos;
					if (cursor_y_pos == height) {
						for (int i = 0; i < (height - 1) * bytesPerLine; ++i) {
							video[i] = video[bytesPerLine+i];
						}
						clearLine(height - 1, 0x0f);
						vramIter -= bytesPerLine;
						--cursor_y_pos;
					}
				}
			}
		}
		updateCursorPos();
	}
};

}
}


#endif // __HW_DEV_CGA_H__
