#include "IDE.h"
#include "PCI.h"
#include "hw/irq/IRQ_IDE1.h"

#include <stdio.h>

namespace {

inline UInt16 ntohs(UInt16 val) {
	return (val >> 8) | (val << 8);
}

}

hw::dev::IDE hw::dev::IDE::inst_;

void hw::dev::IDE::init_regs() {
	const PCIConfigSpace* cs = PCI::Inst().getConfigSpaceEntryByClass(CLASSCODE, SUBCLASSCODE);
	if (!cs) {
		printf("Es wurde kein IDE-Controller gefunden!");
		for (;;);
		return;
	}

	vector_ = cs->interrupt_line;

	if (cs->bar0 & ~1)
		primary_command_.setBaseAddress(cs->bar0 & ~1);
	else
		primary_command_.setBaseAddress(PRIMARY_COMMAND_BLOCK);

	if (cs->bar1 & ~1)
		primary_control_.setBaseAddress(cs->bar1 & ~1);
	else
		primary_control_.setBaseAddress(PRIMARY_CONTROL_BLOCK);

	if (cs->bar2 & ~1)
		secondary_command_.setBaseAddress(cs->bar2 & ~1);
	else
		secondary_command_.setBaseAddress(SECONDARY_COMMAND_BLOCK);

	if (cs->bar3 & ~1)
		secondary_control_.setBaseAddress(cs->bar3 & ~1);
	else
		secondary_control_.setBaseAddress(SECONDARY_CONTROL_BLOCK);

	// TODO: primary vs. secondary
	busmaster_.setBaseAddress(cs->bar4 & ~1);

	UInt8 val = ~primary_command_.cylinder_low.inb();
	primary_command_.cylinder_low.outb(val);
	for (volatile int i = 0; i < 10000; ++i);

	if (primary_command_.cylinder_low.inb() != val) {
		printf("IDE primary master: invalid address register!");
		for (;;);
	}
}

void hw::dev::IDE::init() {
	init_regs();
	//return;

	// TODO: primary vs. secondary


	//                               1          INT
	//primary_control_.control.outb( (1 << 3) | (0 << 1) );

	/*
	UInt8 status = primary_command_.command.inb();
	while ( (status & (1 << 7)) || (!(status & (1 << 6))) ) {
		// warten
		printf("Status: %d\n", status);
		status = primary_command_.command.inb();
	}
	*/

	printf("selecting master device...\n");
	primary_command_.head.outb(0 << 4);
	for (volatile int i = 0; i < 1000000; ++i);

	printf("resetting IDE-Controller...\n");
	primary_control_.control.outb((1 << 2));
	for (volatile int i = 0; i < 1000000; ++i);
	primary_control_.control.outb(0);

	printf("waiting for busybit...\n");
	wait_for_status_bit(BUSY, false);

	printf("waiting for readybit...\n");
	wait_for_status_bit(DRDY, true);

	primary_control_.control.outb(0);

	printf("update...\n");

	updateDeviceInfo();


	if (vector_)
		hw::irq::IRQ_IDE1::init(vector_);
	else
		hw::irq::IRQ_IDE1::init(14);

	hw::irq::IRQ_IDE1::enable();

	printf("finish...\n");
}

UInt64 hw::dev::IDE::calcLBA(UInt16 cylinder, UInt8 head, UInt8 sector) {
	//  linear addr = cylinder * head_count * sector_count + head * sector_count + (sector - 1)
	return cylinder * getDeviceInfo().head_count * getDeviceInfo().sector_count + head * getDeviceInfo().sector_count + (sector - 1);
}

hw::dev::IDE::CHS hw::dev::IDE::calcCHS(unsigned long lba) {
	CHS chs;

	chs.sector   = lba % static_cast<unsigned long>(getDeviceInfo().sector_count) + 1;
	chs.head     = lba / static_cast<unsigned long>(getDeviceInfo().sector_count) % getDeviceInfo().head_count;
	chs.cylinder = lba / (static_cast<unsigned long>(getDeviceInfo().sector_count) * getDeviceInfo().head_count);

	return chs;
}

void hw::dev::IDE::clearBuffer() {
	// read buffer until DRQ cleared or 512 bytes read
	for (int i = 0; (i < 0x100) && status_bit(DRQ); ++i) {
		readblockN(0, 1);
	}
}

// TODO: besser machen
void hw::dev::IDE::updateDeviceInfo() {

	wait_for_status_bit(DRDY, true);
	printf("sending IDENTIFY...\n");
	primary_command_.command.outb(IDENTIFY);

	//readblockN(buffer);

	printf("waiting for BUSY and DRQ...\n");
	wait_for_status_bit(BUSY, false);
	wait_for_status_bit(DRQ, true);

	printf("readblock...\n");
	readblockN(0, 1);
	readblockN(&deviceinfo_.cylinder_count, 1);
	//deviceinfo_.cylinder_count = ntohs(deviceinfo_.cylinder_count);

	printf("readblock...\n");
	readblockN(0, 1);
	readblockN(&deviceinfo_.head_count, 1);
	//deviceinfo_.head_count = ntohs(deviceinfo_.head_count);
	readblockN(0, 2);

	printf("readblock...\n");
	readblockN(&deviceinfo_.sectors_per_track, 1);
	readblockN(0, 3);

	printf("readblock...\n");
	readblockN(&deviceinfo_.serial, 10);
	UInt16* serial16 = (UInt16*) deviceinfo_.serial;
	for (int i = 0; i < 10; ++i) {
		serial16[i] = ntohs(serial16[i]);
	}
	deviceinfo_.serial[20] = 0;
	readblockN(0, 3);

	printf("readblock...\n");
	readblockN(&deviceinfo_.firmware, 4);
	UInt16* firmware16 = (UInt16*) deviceinfo_.firmware;
	for (int i = 0; i < 4; ++i) {
		firmware16[i] = ntohs(firmware16[i]);
	}
	deviceinfo_.firmware[8] = 0;

	printf("readblock...\n");
	readblockN(&deviceinfo_.model, 20);
	UInt16* model16 = (UInt16*) deviceinfo_.model;
	for (int i = 0; i < 20; ++i) {
		model16[i] = ntohs(model16[i]);
	}
	deviceinfo_.model[40] = 0;

	deviceinfo_.sector_count = deviceinfo_.cylinder_count * deviceinfo_.head_count * deviceinfo_.sectors_per_track;

	printf("clearBuffer...\n");
	clearBuffer();
}

void hw::dev::IDE::readblockN(void* data, int words) {
	// waiting for completion

	//printf("Warte auf !BUSY...");
	wait_for_status_bit(BUSY, false);
	//printf(" fertig!\n");

	//printf("Warte auf DRQ...");
	wait_for_status_bit(DRQ, true);

	//printf(" fertig!\n");

	UInt16* data16 = static_cast<UInt16*>(data);

	if (data16)
		for (int i = 0; i < words; ++i)
			data16[i] = primary_command_.data.inw();
	else
		for (int i = 0; i < words; ++i)
			primary_command_.data.inw();
}

void hw::dev::IDE::writeblock(const void* data) {
	// waiting for completion

	wait_for_status_bit(BUSY, false);

	wait_for_status_bit(DRQ, true);


	const UInt16* data16 = static_cast<const UInt16*>(data);
	for (int i = 0; i < 256; ++i) {
		primary_command_.data.outw(data16[i]);
	}
}

//void hw::dev::IDE::setAddress(UInt64 sector) {
//	wait_for_status_bit(BUSY, false);
//
//	primary_command_.sector_count.outb(1);
//	primary_command_.sector_number.outb(sector & 0xff);
//	primary_command_.cylinder_low.outb((sector >> 8) & 0xff);
//	primary_command_.cylinder_high.outb((sector >> 16) & 0xff);
//	primary_command_.head.outb((sector >> 24) & 0x0f | (7 << 5));
//}


//void hw::dev::IDE::setAddress48(UInt64 sector) {
//	wait_for_status_bit(BUSY, false);
//
//	primary_command_.sector_count.outb(1);
//
//	primary_command_.sector_number.outb(extract_byte(sector, 3));
//	primary_command_.sector_number.outb(extract_byte(sector, 0));
//
//	primary_command_.cylinder_low.outb(extract_byte(sector, 4));
//	primary_command_.cylinder_low.outb(extract_byte(sector, 1));
//
//	primary_command_.cylinder_high.outb(extract_byte(sector, 5));
//	primary_command_.cylinder_high.outb(extract_byte(sector, 2));
//
//	primary_command_.head.outb((1 << 6));
//}

// reads 512-Byte block
// TODO: primary vs. secondary
//void hw::dev::IDE::readblock(void* data, UInt8 sector, UInt16 cylinder, UInt8 head) {
//
//	wait_for_status_bit(BUSY, false);
//
//	primary_command_.sector_count.outb(1);
//	primary_command_.sector_number.outb(sector);
//	primary_command_.cylinder_low.outb(cylinder & 0xff);
//	primary_command_.cylinder_high.outb(cylinder >> 8);
//	primary_command_.head.outb(head | (5 << 5));
//
//	//primary_control_.control.outb( (1 << 3) | (0 << 1) );
//
//	wait_for_status_bit(DRDY, true);
//
//	primary_command_.command.outb(READ);
//	readblockN(data);
//}

//void hw::dev::IDE::readblock(void* data, UInt64 sector) {
//
//	setAddress(sector);
//
//	//primary_control_.control.outb( (1 << 3) | (0 << 1) );
//
//	wait_for_status_bit(DRDY, true);
//
//	primary_command_.command.outb(READ);
//	readblockN(data);
//}

//void hw::dev::IDE::readblock48(void* data, UInt64 sector) {
//
//	setAddress48(sector);
//
//	wait_for_status_bit(DRDY, true);
//
//	primary_command_.command.outb(READ_EXT);
//	readblockN(data);
//}

// writes 512-Byte block
//void hw::dev::IDE::writeblock(const void* data, UInt8 sector, UInt16 cylinder, UInt8 head) {
//
//}

//void hw::dev::IDE::writeblock(const void* data, UInt64 sector) {
//
//	setAddress(sector);
//
//	wait_for_status_bit(DRDY, true);
//
//	primary_command_.command.outb(WRITE);
//	writeblock(data);
//}

//void hw::dev::IDE::writeblock48(const void* data, UInt64 sector) {
//
//	setAddress48(sector);
//
//	wait_for_status_bit(DRDY, true);
//
//	primary_command_.command.outb(WRITE_EXT);
//	writeblock(data);
//}
//
//void hw::dev::IDE::setDMABuffer(void* address, UInt16 size) {
//	prdt_[0].address = ((unsigned long int) address) & 0xFFFFFFFF;
//	prdt_[0].flags = 0x80; // last PRD
//	prdt_[0].size = size;
//}

//void hw::dev::IDE::writePRD(bool enable, bool write) {
//	busmaster_.command.outb((enable << 0) | (write << 3));
//	busmaster_.address.outl( ((unsigned long int) &prdt_[0]) & 0xFFFFFFFF );
//}

//UInt8 hw::dev::IDE::getDMAStatus() {
//	return busmaster_.status.inb();
//}

//void hw::dev::IDE::readblockdma48(void* data, UInt64 sector) {
//
//	setAddress48(sector);
//
//	wait_for_status_bit(DRDY, true);
//
//	setDMABuffer(data, 512);
//	writePRD(true, false);
//	primary_command_.command.outb(READ_DMA_EXT);
//}
