#ifndef __HW_DEV_PIC_H__
#define __HW_DEV_PIC_H__

#include "stdinc.h"
#include "IOPort.h"

namespace hw {
namespace dev {

struct PIC_IRQ {
	union {
		struct {
			bool timer    :1;
			bool keyboard :1;
			bool pic2     :1;
			bool com2     :1;
			bool com1     :1;
			bool irq5     :1;
			bool floppy   :1;
			bool lpt1     :1;

			bool CMOS     :1;
			bool IRQ9     :1;
			bool IRQ10    :1;
			bool IRQ11    :1;
			bool MOUSE    :1;
			bool NUMCO    :1;
			bool IDE1     :1;
			bool IDE2     :1;
		};

		UInt16 mask;
	};

	PIC_IRQ(UInt16 m = 0) : mask(m) {}
};

class PIC {
private:
	IOPort imr1;
	IOPort imr2;

	UInt16 irq_mask;

	static PIC _thePIC;

public:
	enum IRQ {
		TIMER    = 1 << 0,
		KEYBOARD = 1 << 1,
		PIC2     = 1 << 2,
		COM2     = 1 << 3,
		COM1     = 1 << 4,
		IRQ5     = 1 << 5,
		FLOPPY   = 1 << 6,
		LPT1     = 1 << 7,

		CMOS     = 1 << 8,
		IRQ9     = 1 << 9,
		IRQ10    = 1 << 10,
		IRQ11    = 1 << 11,
		MOUSE    = 1 << 12,
		NUMCO    = 1 << 13,
		IDE1     = 1 << 14,
		IDE2     = 1 << 15
	};

public:
	void init() {
		imr1.setAddress(0x21);
		imr2.setAddress(0xa1);
		irq_mask = 0xfb | (0xff << 8);
	}

	static PIC& Inst() { return _thePIC; }

public:
	void setIRQMask(PIC_IRQ mask);

	PIC_IRQ getIRQMask() { return PIC_IRQ(irq_mask); }

	void enableIRQ(UInt8 irq) {
		irq_mask &= ~(1 << irq);
		setIRQMask(irq_mask);
	}

	void disableIRQ(UInt8 irq) {
		irq_mask |= (1 << irq);
		setIRQMask(irq_mask);
	}

};

}
}


#endif // __HW_DEV_PIC_H__
