/*
 * VirtioNet.cpp
 *
 *  Created on: Mar 8, 2010
 *      Author: bossel00
 */

#include "VirtioNet.h"
#include "PCI.h"

#include <stdio.h>

// TODO: Hersteller- und Gerätenummern konfigurierbar machen
#define VIRTIO_NET_VENDOR 0x1AF4
#define VIRTIO_NET_DEVICE 0x1000

namespace hw {
namespace dev {

VirtioNet VirtioNet::inst_;

void VirtioNet::init() {
	const PCIConfigSpace* cs = PCI::Inst().getConfigSpaceEntry(VIRTIO_NET_DEVICE, VIRTIO_NET_VENDOR);
	if (!cs)
		return;

	vector = cs->interrupt_line;

	io_.setBaseAddress(cs->bar0 & 0xFFFFFFFC);

	// clear vrings (zeroing out)
	for (unsigned long* ptr = (unsigned long*) transmit_ring_.desc; ptr != transmit_ring_.end; *ptr++ = 0);
	for (unsigned long* ptr = (unsigned long*)  receive_ring_.desc; ptr !=  receive_ring_.end; *ptr++ = 0);

	// build ringbuffer
	for (int i = 0; i < RING_SIZE - 1; ++i) {
		transmit_ring_.desc[i].flags = 0;
		transmit_ring_.desc[i].next = i + 1;
		receive_ring_ .desc[i].flags = 0;
		receive_ring_ .desc[i].next = i + 1;
	}
	transmit_ring_.desc[RING_SIZE - 1].next = 0;
	receive_ring_ .desc[RING_SIZE - 1].next = 0;

	// mark unused used-descriptors
	//for (int i = 0; i < RING_SIZE; ++i) {
	//	transmit_ring_.used.ring[i].id = -1;
	//}

	transmit_ring_.last_used = 0;
	transmit_ring_.next_free = 0;
	transmit_ring_.next_avail = 0;

	receive_ring_.last_used = 0;
	receive_ring_.next_free = 0;
	receive_ring_.next_avail = 0;

	for (int i = 0; i < RING_SIZE/2-1; ++i) {
		int id = receive_ring_.get_next_desc();

		receive_ring_.desc[id].addr = (unsigned long) &receive_buffer_[i].net_hdr;
		receive_ring_.desc[id].len = sizeof(receive_buffer_[i].net_hdr);
		receive_ring_.desc[id].flags = vring_desc::VRING_DESC_F_NEXT | vring_desc::VRING_DESC_F_WRITE;

		receive_ring_.desc[id + 1].addr  = (unsigned long) &receive_buffer_[i].packet;
		receive_ring_.desc[id + 1].len   = sizeof(receive_buffer_[i].packet);
		receive_ring_.desc[id + 1].flags = vring_desc::VRING_DESC_F_WRITE;

		receive_ring_.add_desc(id); // ready to receive
	}

	net_hdr_none.flags = 0;
	net_hdr_none.gso_type = virtio_net_hdr::VIRTIO_NET_HDR_GSO_NONE;
	net_hdr_none.hdr_len = 0;
	net_hdr_none.gso_size = 0;
	net_hdr_none.csum_start = 0;
	net_hdr_none.csum_offset = 0;

	// Reset Virtio-Device
	io_.status.outb(0);

	// ack
	getISR();

	// setup receive descriptor ring
	io_.vring_queue_selector.outw(VIRTIO_RECEIVE_QUEUE_SELECTOR);
	io_.vring_page_num.outl( ((unsigned long) &receive_ring_.desc) / 4096 );

	// setup transmit descriptor ring
	io_.vring_queue_selector.outw(VIRTIO_TRANSMIT_QUEUE_SELECTOR);
	io_.vring_page_num.outl( ((unsigned long) &transmit_ring_.desc) / 4096 );

	io_.guest_features.outl(1 << VIRTIO_NET_F_MAC);
	io_.status.outb(VIRTIO_CONFIG_S_DRIVER | VIRTIO_CONFIG_S_DRIVER_OK);

	notify(VIRTIO_RECEIVE_QUEUE_SELECTOR);
}

int VirtioNet::vring::get_next_desc() {
	int next = next_free;
	next_free = (next_free+2) % RING_SIZE;
	last_used++;
	return next;
}

void VirtioNet::vring::add_desc(int desc) {
	avail.ring[next_avail] = desc;
	//avail.ring[((avail.idx * 2) + 1) % RING_SIZE] = desc+1;

	next_avail = (next_avail+1) % RING_SIZE;

	//avail.idx = (avail.idx+1) % RING_SIZE;
	avail.idx++;
}

UInt64 VirtioNet::getDefaultAddress() {
	UInt64 address = 0;
	address |= io_.config.offset(0).inb() << 40;
	address |= io_.config.offset(1).inb() << 32;
	address |= io_.config.offset(2).inb() << 24;
	address |= io_.config.offset(3).inb() << 16;
	address |= io_.config.offset(4).inb() <<  8;
	address |= io_.config.offset(5).inb() <<  0;

	return address;
}

void VirtioNet::send(const void* data, unsigned datasize) {
	while(transmit_ring_.empty()) {
		// trigger host
		notify(VIRTIO_TRANSMIT_QUEUE_SELECTOR);
		//io_.pci_isr.inb();
	}

	int id = transmit_ring_.get_next_desc();

	transmit_ring_.desc[id].addr = (unsigned long) &net_hdr_none;
	transmit_ring_.desc[id].len = sizeof(net_hdr_none);
	transmit_ring_.desc[id].flags = vring_desc::VRING_DESC_F_NEXT;

	transmit_ring_.desc[id + 1].addr  = (unsigned long) data;
	transmit_ring_.desc[id + 1].len   = datasize;
	transmit_ring_.desc[id + 1].flags = 0;

	transmit_ring_.add_desc(id); // ready to send

	// trigger host
	// TODO: später triggern
	//notify(VIRTIO_TRANSMIT_QUEUE_SELECTOR);
	//io_.pci_isr.inb();
}

bool VirtioNet::hasBeenSent(const void* data) {
	// TODO: Implementieren
}

unsigned VirtioNet::receive(void* data, unsigned datasize) {
	while (!packet_received()) {
		// wait
	}

	int id = receive_ring_.get_next_desc();

	UInt8* ptr = static_cast<UInt8*> (data);

	for (unsigned i = 0; i < datasize && i < sizeof(ethernet_packet); ++i) {
		ptr[i] = ((const UInt8*) &receive_buffer_[(id/2 + 1) % (RING_SIZE/2)].packet)[i];
	}

	receive_ring_.desc[id].addr = (unsigned long) &receive_buffer_[id/2].net_hdr;
	receive_ring_.desc[id].len = sizeof(virtio_net_hdr);
	receive_ring_.desc[id].flags = vring_desc::VRING_DESC_F_NEXT | vring_desc::VRING_DESC_F_WRITE;

	receive_ring_.desc[id + 1].addr  = (unsigned long) &receive_buffer_[id/2].packet;
	receive_ring_.desc[id + 1].len   = sizeof(ethernet_packet);
	receive_ring_.desc[id + 1].flags = vring_desc::VRING_DESC_F_WRITE;

	receive_ring_.add_desc(id); // ready to receive
}

const void* VirtioNet::nextData() {
	// TODO: Implementieren
}

unsigned nextDataLength() {
	// TODO: Implementieren
}

void nextFree() {
	// TODO: Implementieren
}

}
}
