/** \author Christian Meier
 *  \version $Id: InterruptScheme.h 995 2009-08-11 14:26:31Z mike $
 */

#ifndef HW_DEV_CAN_MULTICAN_INTERRUPT_SCHEME_H
#define HW_DEV_CAN_MULTICAN_INTERRUPT_SCHEME_H

/* Notes to the usage of interrupt lines:
 *  Due to the requirement to be able to separatly disable controller interrupts, but still keeping
 *  the possibility to deduce the HwObject from interrupt line and pending registers in constant time
 *  it is neccessary to assign at least 1 Interrupt line for each controller.
 *  To be able to make further enchancements I decided to use different interrupt lines for receive
 *  transmit interrupts.
 *  
 *  Also the driver makes a difference between ControllerId and ControllerUnit. This hasn't to be
 *  made here because the interrupt requests of the controller units are made by mapping the
 *  ControllerIds on the interrupt lines as the scheme below and not ControllerUnits.
 */
#define HW_DEV_CAN_MULTICAN_INT_BASE(ControllerId)			\
				((ControllerId) << 2)
				
#define HW_DEV_CAN_MULTICAN_RX_INT(ControllerId)				\
				(((ControllerId) << 2) + 0)

#define HW_DEV_CAN_MULTICAN_TX_INT(ControllerId)				\
				(((ControllerId) << 2) + 1)

#define HW_DEV_CAN_MULTICAN_ALERT_INT(ControllerId)			\
				(((ControllerId) << 2) + 2)


#define CAN_SRC_RX_ID0            hw::irq::CAN_SRC0
#define CAN_SRC_RX_ID0_HANDLER()  "% hw::irq::CAN_SRC0::Handler()"

#define CAN_SRC_TX_ID0           	hw::irq::CAN_SRC1
#define CAN_SRC_TX_ID0_HANDLER()  "% hw::irq::CAN_SRC1::Handler()"

#define CAN_SRC_ALERT_ID0            hw::irq::CAN_SRC2
#define CAN_SRC_ALERT_ID0_HANDLER()  "% hw::irq::CAN_SRC2::Handler()"


#define CAN_SRC_RX_ID1            hw::irq::CAN_SRC4
#define CAN_SRC_RX_ID1_HANDLER()  "% hw::irq::CAN_SRC4::Handler()"

#define CAN_SRC_TX_ID1            hw::irq::CAN_SRC5
#define CAN_SRC_TX_ID1_HANDLER()  "% hw::irq::CAN_SRC5::Handler()"

#define CAN_SRC_ALERT_ID1            hw::irq::CAN_SRC6
#define CAN_SRC_ALERT_ID1_HANDLER()  "% hw::irq::CAN_SRC6::Handler()"


#define CAN_SRC_RX_ID2            hw::irq::CAN_SRC8
#define CAN_SRC_RX_ID2_HANDLER()  "% hw::irq::CAN_SRC8::Handler()"

#define CAN_SRC_TX_ID2            hw::irq::CAN_SRC9
#define CAN_SRC_TX_ID2_HANDLER()  "% hw::irq::CAN_SRC9::Handler()"

#define CAN_SRC_ALERT_ID2            hw::irq::CAN_SRC10
#define CAN_SRC_ALERT_ID2_HANDLER()  "% hw::irq::CAN_SRC10::Handler()"


#define CAN_SRC_RX_ID3            hw::irq::CAN_SRC12
#define CAN_SRC_RX_ID3_HANDLER()  "% hw::irq::CAN_SRC12::Handler()"

#define CAN_SRC_TX_ID3            hw::irq::CAN_SRC13
#define CAN_SRC_TX_ID3_HANDLER()  "% hw::irq::CAN_SRC13::Handler()"

#define CAN_SRC_ALERT_ID3            hw::irq::CAN_SRC14
#define CAN_SRC_ALERT_ID3_HANDLER()  "% hw::irq::CAN_SRC14::Handler()"


namespace hw { namespace dev { namespace can { namespace multican {

enum InterruptType {
	RX = 0,
	TX = 1,
	ALERT = 2
};

}}}}

#endif /* HW_DEV_CAN_MULTICAN_INTERRUPT_SCHEME_H */
