// $Id: In1.h 995 2009-08-11 14:26:31Z mike $
// Created:   06.06.2006 lohmann

/**
 * @file In1.h
 * @brief TC1796 version of hw::hal::In1
 */

#ifndef __HW_HAL_In1_H__
#define __HW_HAL_In1_H__

#include "stdinc.h"
#include "hw/dev/P1.h"
 
namespace hw { namespace hal {
 
/**
 * @brief Digital input line 0
 * 
 * The digital input line of In1 is typically connected to some button.
 * In1 may be extended by an aspect (In1IRQ.ah) to trigger IRQs on a rising or
 * falling edge. 
 */
class In1 {
	static In1 theIn1_;
	//! Performs start-time initializations. Only called by advice.
	void init() {
		port().setMode( PORT_BIT, Port::INPUT | Port::INPUT_PULLUP );
	}
  protected:
    In1(){}
  public:

	//! Returns the one-and-only In1 instance
	static In1& Inst() {
		return theIn1_;
	}
  	
  	//! The internal hw::dev used to implement this line. Currently fixed to P1
  	typedef hw::dev::P1 Port;
  	
  	//! The used bit number of the internal hw::dev port
  	enum DOXY_SYMBOL(Consts){ PORT_BIT = cfHAL_IN1_BIT };
  	
  	//! Provides access to the internally used port object
  	const Port& port() const{
  		return Port::Inst();
  	}
  	Port& port() {
  		return Port::Inst();
  	}
	
	//! Returns the level of the input line: low == 0, high != 0
	CIAO_STD Bool get() const {
		return port().get( PORT_BIT );
	};
	//! Returns the level of the input line: low == 0, high != 0
	CIAO_STD operator Bool() const {
		return get();
	}
	
};

}}	// namespace hw::hal

 
#endif // __HW_HAL_In1_H__
