// $Id: TCBBasic.h 995 2009-08-11 14:26:31Z mike $
// Created:   19.05.2006 lohmann

/**
 * @file TCBBasic.h
 * @brief Contains the definitions of a simply dispatchable Coroutine implementation.
 * 
 */


#ifndef __HW_HAL_TCB_H__
#define __HW_HAL_TCB_H__

#include "Continuation.h"	

/// Thread functions must obey the following (architecure-dependend) attributes
#define cfHAL_THREADFUNC_ATTRIBUTES 	// nothing special on tricore


namespace hw { namespace hal { 


class TCBBasic;

class TCBBasicBase : public Continuation {
public:
	typedef void (* cfHAL_THREADFUNC_ATTRIBUTES ThreadFunc)(TCBBasic* me);

protected:
	typedef Continuation Super;		
	ThreadFunc			start_;		// start address of thread
	void**				tos_;		// initial top of stack

	/**
	 * \brief The internal thread start function. 
	 * 
	 */
	static void kickoff( TCBBasic* me );
};

class TCBBasic : public TCBBasicBase {
  public:
	typedef TCBBasicBase Super;		
	
	void init( void** tos, ThreadFunc startAddress );

		
	void reset() {
		Super::init();
	}

	/**
	 * \brief switches to a non-new TCBBasic.
	 * 
	 */
	void CIAO_INLINE saveAndSwitchto( TCBBasic* to ) {
		Super::saveAndSwitchto( to );
	} 

	void CIAO_INLINE saveAndStart( TCBBasic* to ) {
		Super::saveAndStart( to->tos_, (StartFunc) kickoff, to );
	}
	
	void CIAO_INLINE go() {
		Super::go( tos_, (StartFunc) kickoff );
	}
	 
	void CIAO_INLINE start(TCBBasic* to) {
		Super::start( to->tos_, (StartFunc) kickoff, to );
	} 

	void CIAO_INLINE  activate( TCBBasic* to ) {
		hw::JUMP2( activate_internal, this, to );
	}

private:
	static void activate_internal( TCBBasic* from, TCBBasic* to );

public:	
	
	void dispatch( TCBBasic* to ) {
		if( to->hasSeenCPU() ) {
			saveAndSwitchto( to );
		}
		else {
			saveAndStart( to );
		}
	} 			
	 
};
	
} } // namespace hw::hal


#endif // __HW_HAL_TCB_H__
