/**
 * \file OutputDeviceCGA.h
 * \brief Implements a low-level interface to CGA display
 *
 */

#ifndef __HW_HAL_OUTPUTDEVICECGA__
#define __HW_HAL_OUTPUTDEVICECGA__

#include "stdinc.h"

namespace hw { namespace hal {

class OutputDeviceCGA {
  public:
	typedef UInt8 Attribute;

	static const Attribute DEFAULT_ATTRIBUTE = 7;

  private:
	static OutputDeviceCGA theOutputDevice_;

	Attribute attribute_;

  public:

	void init() {
		resetAttribute();
		clearScreen();
	}

  	static OutputDeviceCGA& Inst() {
  		return theOutputDevice_;
  	}

  	void clearScreen();

  	void setCursorPosition(int x, int y);

  	void putCharacter(char c);
  	void putString(const char* str, unsigned long size);

  	void setAttribute(const Attribute& attribute);

  	void resetAttribute(); // reset to default

  private:
	OutputDeviceCGA() {}
};

}} // namespace hw::hal



#endif // __HW_HAL_OUTPUTDEVICECGA__
