/** \author Christian Meier
 *  \version $Id: RxHwObjectType.h 995 2009-08-11 14:26:31Z mike $
 */

#ifndef HW_HAL_CAN_RX_HW_OBJECT_TYPE_H
#define HW_HAL_CAN_RX_HW_OBJECT_TYPE_H

#include "os/com/ComStack_Types.h"

#include "RxHwObjectConfigType.h"

namespace hw { namespace hal { namespace can {

class RxLpduType;

/**
 *	This is the base class for all rx mappings
 *  It mustn't be used directly. It is intended to be a base class.
 *  The attributes must be set before calling Init.
 */
class RxHwObjectType {
public:
	RxLpduType *
	GetLpduByCanId(hw::dev::can::IdType CanId) {
		if(RxLpdus_ != NULL) {
			return GetLpduByCanId_NotNull(CanId);
		} else {
			return NULL;
		}
	}

protected:
	RxHwObjectType () {}
	
	/**
	 *  The Init method initializes the ALREADY PROVIDED array of RxLpdu Pointers
	 *  with the correct data.
	 */
	void
	Init(RxHwObjectConfigType const * RxHwObjectConfig);
	
	// Has to be set by subclass !!!
	RxLpduType * * RxLpdus_; // Array of pointers to RxLpdus
	RxLpduIdStorageType RxLpduMultiplicity_;

private:
	/**
	 *  The method is the internal version of GetLpduByCanId where it is 
	 *  assured that the member RxLpdus_ is not NULL. This happens, when
	 *  the underlying HwObject is a transmit object an not an receive object.
	 */
	RxLpduType *
	GetLpduByCanId_NotNull(hw::dev::can::IdType CanId) {
		// The simple case is not to use software filter.
		// we simply take the first entry
		return RxLpdus_[0];
	}
};

}}}

#endif
