// // Created:   14.01.2008 sirupfis

#include "PCP_asm.inc"

//-----------------------------------------------//
// Note:
// A special section name '.pcptext' is used to
// direct the linker/locator to locate this
// section in pcp code. The CODE and INIT
// attributes are used to force this section to
// be copied at the start-up of the program to
// 'pcode' memory.
//-----------------------------------------------//
//            .sdecl ".pcptext", CODE, INIT	//	<- funktioniert mit unserem Compiler nicht ...
//            .sect ".pcptext"

//Macro for switching to a certain page in PRAM
//tmpreg: 		temporary register needed to build mask
//toPageMask:	mask that contains the page you want to switch to in bits [8:15]. Remaining bits have to be zeros
.macro SWITCH_PRAM_PAGE tmpreg, toPageMask
				ldl.il \tmpreg , 0xff			// Set Mask for following and; leave sregbits[7 : 0] on r7 untouched
				ldl.iu \tmpreg , 0x0			// 
				and r7, \tmpreg , CC_UC			// Change dptr to page 5
				ldl.il \tmpreg , \toPageMask	// Set Mask for following or
				or r7, \tmpreg , CC_UC			// Write new pagenumber to sreg
.endm

//Macros for syncronization with main CPU

//Peterson's algorithm:
//flag1 = true;
//turn = 0;
//while (flag0 && (turn == 0)) {} // busy waiting
// <kritischer Abschnitt>
//flag1 = false;

.macro ENTER_SYNC tmpreg
// set flag1
				ldl.il \tmpreg, 0x1				// set flag1 = 1
				ldl.iu \tmpreg, 0x0
				st.pi \tmpreg, 0x1
// set turn to 0
				ldl.il \tmpreg, 0x0				// set turn = 0
				ldl.iu \tmpreg, 0x0
				st.pi \tmpreg, 0x2
// wait till flag0 is false and turn is 0
wait_loop\@:
				ld.pi \tmpreg, 0x0				// flag0 == 1 ?
				comp.i \tmpreg, 0x1
				jc.a end_wait\@, cc_NZ

				ld.pi \tmpreg, 0x2				// turn == 0 ?
				comp.i \tmpreg, 0x0
				jc.a end_wait\@, cc_NZ
				
				jc.a wait_loop\@, cc_UC
end_wait\@:
.endm

.macro LEAVE_SYNC tmpreg
// reset flag1						
				ldl.il \tmpreg, 0x0				// set flag1 = 0
				ldl.iu \tmpreg, 0x0
				st.pi \tmpreg, 0x1
				
.endm

.macro SET_P0_PIN tmpreg, pin
				ldl.il \tmpreg, 0xf0000c00 
				ldl.iu \tmpreg, 0xf0000c00 
				set.f [\tmpreg], \pin, size=32
				nop
.endm

.macro CLEAR_P0_PIN tmpreg, pin
				ldl.il \tmpreg, 0xf0000c00 
				ldl.iu \tmpreg, 0xf0000c00 
				clr.f [\tmpreg], \pin, size=32
				nop
.endm

.pcptext													//	<- ... aber so gehts und wird auch vom Startupcode kopiert

		nop							// nops for channel 0, which is never used
		nop							// but it needs also space in this table
CH1:	JC.A channel_x, cc_UC 	// Channel 1
CH2:	JC.A channel_x, cc_UC 	// Channel 2
CH3:	JC.A channel_x, cc_UC 	// Channel 3
CH4:	JC.A channel_x, cc_UC 	// Channel 4
CH5:	JC.A channel_x, cc_UC 	// Channel 5
CH6:	JC.A channel_x, cc_UC 	// Channel 6
CH7:	JC.A channel_x, cc_UC 	// Channel 7
CH8:	JC.A channel_x, cc_UC 	// Channel 8
CH9:	JC.A channel_x, cc_UC 	// Channel 9
CH10:	JC.A channel_x, cc_UC 	// Channel 10
CH11:	JC.A channel_x, cc_UC 	// Channel 11
CH12:	JC.A channel_x, cc_UC 	// Channel 12
CH13:	JC.A channel_x, cc_UC 	// Channel 13
CH14:	JC.A channel_x, cc_UC 	// Channel 14
CH15:	JC.A channel_x, cc_UC 	// Channel 15
CH16:	JC.A channel_x, cc_UC 	// Channel 16
CH17:	JC.A channel_x, cc_UC 	// Channel 17
CH18:	JC.A channel_x, cc_UC 	// Channel 18
CH19:	JC.A channel_x, cc_UC 	// Channel 19
CH20:	JC.A channel_x, cc_UC 	// Channel 20
CH21:	JC.A channel_x, cc_UC 	// Channel 21
CH22:	JC.A channel_x, cc_UC 	// Channel 22
CH23:	JC.A channel_x, cc_UC 	// Channel 23
CH24:	JC.A channel_x, cc_UC 	// Channel 24
CH25:	JC.A channel_x, cc_UC 	// Channel 25
CH26:	JC.A channel_x, cc_UC 	// Channel 26
CH27:	JC.A channel_x, cc_UC 	// Channel 27
CH28:	JC.A channel_x, cc_UC 	// Channel 28
CH29:	JC.A channel_x, cc_UC 	// Channel 29
CH30:	JC.A channel_x, cc_UC 	// Channel 30
CH31:	JC.A channel_x, cc_UC 	// Channel 31
            
channel_x:									// one channel programm for all interrupts
//				SET_P0_PIN tmpreg=r0, pin=0x8
//				CLEAR_P0_PIN tmpreg=r0, pin=0x8

				set r7,0x5					// allow interrupts

// load address of ready field to register r1

				ldl.il r0, 0x4				// load address of ready field
				ldl.iu r0, 0x0
				ld.p r1,[r0], cc_UC			// to register r1
				
// load current priority number to r2
				
				mov r2, r6, cc_UC
				shr r2, 0x8
				shr r2, 0x8
				shr r2, 0x8					


// task table spans over 2 memory pages (5 and 6), we need to handle that

// calculate array index and switch to page 6 if needed				
				ldl.il r0, 0x1				// Array starts in page 5 at position 1
				ldl.iu r0, 0x0				//
				add r0, r2, CC_UC			// find correct entry for interrupt
				add r0, r2, CC_UC			// 
				add r0, r2, CC_UC			// r0 now contains offset of the current entry inside the tasktable
				comp.i r0, 63				// offset greater than page size?
				jc in_page_5, CC_SLE 
in_page_6:									// Entry resides in page 6
// switch to PRAM page 6
			    SWITCH_PRAM_PAGE tmpreg=r3, toPageMask=0x0600			    
			   	sub.i r0, 63				// correct offset in r0; first element in page 6 starts at offset 0x0
				jc read_tasklist_entry, CC_UC		
in_page_5:									// Entry resides in page 5, offset in r0 is ok
// switch to PRAM page 5
				SWITCH_PRAM_PAGE tmpreg=r3, toPageMask=0x0500
read_tasklist_entry:
// load task priority to r3 				
				 	
				ld.p r3,[r0], cc_UC			// load task prio to r3
				
// load task pointer to r4
				
				add.i r0, 0x1
				ld.p r4,[r0], cc_UC		

// load task number to r5

				add.i r0, 0x1
				ld.p r5,[r0], cc_UC		
				
// switch back to PRAM page 4
				SWITCH_PRAM_PAGE tmpreg=r0, toPageMask=0x0400

//Stop other PCP programs from interrupting
				clr r7,0x5					// block interrupts
				nop							// this nop is needed
//Sync with main CPU				
				ENTER_SYNC tmpreg=r0

// ***Start of Critical Section***

// set ready bit of task priority in the ready field
				ld.f r0, [r1], size=32		// load ready field to r0
				set r7, 0x2					// set carry flag
				inb r0, r3, cc_UC			// set ready bit for task
				st.f r0, [r1], size=32		// restore ready field		
					
// store the task number to tasks_ in TasklistTypes
				add.i r1, 0x4				// store task number to task_
				add r1, r3, cc_UC
				add r1, r3, cc_UC
				add r1, r3, cc_UC
				add r1, r3, cc_UC
 				st.f r5,[r1], size=32
 				
// set task state to ready 								
//				ld.f r5, [r4], size=32
//				mov r0, r5, cc_UC			// save old task state to r0 - needed later
//				ldl.il r2,0xfffff0ff		// set task state to ready
//				ldl.iu r2,0xfffff0ff
//				and r5, r2, cc_UC
//				ldl.il r2,0x00000200
//				ldl.iu r2,0x00000200
//				or r5, r2, cc_UC
//				st.f r5,[r4], size=32
// Leave Sync with main CPU							
				LEAVE_SYNC tmpreg=r5
// Allow interruptions by higher priority PCP programs
				set r7,0x5					// allow interrupts to check if higher interrupts are pending



// load need reschedule field address to register r5

				ldl.il r4, 0x6
				ldl.iu r4, 0x0
				ld.p r5,[r4], cc_UC	
					
				ldl.il r4, 0x1				// needreschedule mask
				ldl.iu r4, 0x0

// Lock again before deciding between AST or no AST				
				clr r7,0x5					// block interrupts
				nop							// this nop is needed
//				ENTER_SYNC tmpreg=r2

// If priority field >= 2^(i+1) then we were interrupted 
// by a higher priority interrupt, hence no rescheduling necessary (i=priority of my interrupt)
				mov r0, r3, cc_UC
				add.i r0, 0x1
				ldl.il r1, 0x0
				ldl.iu r1, 0x0
				set r7, 0x2					// set carry flag
				inb r1, r0, cc_UC			// set the bit depicted by content of r0 inside r1 to carry flag
											// r1 is now 2^(i+1)
				ldl.il r2, 0x4				// load address of ready field
				ldl.iu r2, 0x0
				ld.p r0,[r2], cc_UC			// to r0
				ld.f r2, [r0], size=32		// load ready field to r2
				comp r2, r1, cc_UC			// compare 2^(i+1) to ready field
				
// if there is a higher priority process ready, we were interrupted, so goto exit without interrupt...
				
				jc.a	exit_, cc_UGE
// now we still need to test if there's a higher priority process already running on the processor
				ldl.il r0, 0x3				// load address of address of cpu priority field
				ldl.iu r0, 0x0
				ld.p r1,[r0], cc_UC			// load cpu priority to r1
				comp r1,r3,cc_UC			// subtract task prio from cpu prio
				jc.a	exit_, cc_NN		// if cpu prio > task prio goto exit without interrupt...	
				
// ... otherwise set needreschedule
						
				st.f r4,[r5], size=32		// set needreschedule
				
// if cpu was in idle loop goto exit without interrupt ...
				ldl.il r0, 0x3				// load address of address of cpu priority field
				ldl.iu r0, 0x0
				ld.p r1,[r0], cc_UC			// load cpu priority to r1
				comp.i r1, 0x0 				// if cpu in idleloop, dont send interrupt
				jc.a exit_, cc_Z

// ... otherwise exit and send interrupt to cpu to trigger an AST							
				exit ec=0, st=0, int=1, ep=0, cc_UC	// exit, send int to cpu if int prio > cpu prio
				JC.A channel_x, cc_UC
				
// exit without interrupt
				
exit_:      
				exit ec=0, st=0, int=0, ep=0, cc_UC	// exit, send no interrupt
				JC.A channel_x, cc_UC

				
//-----------------------------------------------//
// Note:
// A special section name '.pcpdata' is used to
// direct the linker/locator to locate this
// section in pcp data memory (pram). This
// section is initialized at start-up time.
//-----------------------------------------------//

.pcpdata

ch0:	// never used, but needed ...
           .word 0x0							// R7
           .word 0x0                            // R6
           .word 0x0                            // R5
           .word 0x0                            // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0
ch1:
           .word 0x0460                     	// R7 channel enabled
           .word 0x01010000		               	// R6 CPU Interrupt with prio 1, next run with prio lvl 1
           .word 0x0                            // R5
           .word 0x0				            // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch2:
           .word 0x0460                     // R7 channel enabled
           .word 0x02010000							// R6 CPU Interrupt with prio 1, next run with prio lvl 2
           .word 0x0                            // R5
           .word 0x0					               // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch3:
           .word 0x0460                     // R7 channel enabled
           .word 0x03010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch4:
           .word 0x0460                     // R7 channel enabled
           .word 0x04010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch5:
           .word 0x0460                     // R7 channel enabled
           .word 0x05010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch6:
           .word 0x0460                     // R7 channel enabled
           .word 0x06010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch7:
           .word 0x0460                     // R7 channel enabled
           .word 0x07010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch8:
           .word 0x0460                     // R7 channel enabled
           .word 0x08010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch9:
           .word 0x0460                     // R7 channel enabled
           .word 0x09010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch10:
           .word 0x0460                     // R7 channel enabled
           .word 0x0a010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch11:
           .word 0x0460                     // R7 channel enabled
           .word 0x0b010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch12:
           .word 0x0460                     // R7 channel enabled
           .word 0x0c010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch13:
           .word 0x0460                     // R7 channel enabled
           .word 0x0d010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch14:
           .word 0x0460                     // R7 channel enabled
           .word 0x0e010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch15:
           .word 0x0460                     // R7 channel enabled
           .word 0x0f010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch16:
           .word 0x0460                     // R7 channel enabled
           .word 0x10010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch17:
           .word 0x0460                     // R7 channel enabled
           .word 0x11010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch18:
           .word 0x0460                     // R7 channel enabled
           .word 0x12010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch19:
           .word 0x0460                     // R7 channel enabled
           .word 0x13010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch20:
           .word 0x0460                     // R7 channel enabled
           .word 0x14010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch21:
           .word 0x0460                     // R7 channel enabled
           .word 0x15010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch22:
           .word 0x0460                     // R7 channel enabled
           .word 0x16010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch23:
           .word 0x0460                     // R7 channel enabled
           .word 0x17010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch24:
           .word 0x0460                     // R7 channel enabled
           .word 0x18010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch25:
           .word 0x0460                     // R7 channel enabled
           .word 0x19010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch26:
           .word 0x0460                     // R7 channel enabled
           .word 0x1a010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch27:
           .word 0x0460                     // R7 channel enabled
           .word 0x1b010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch28:
           .word 0x0460                     // R7 channel enabled
           .word 0x1c010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch29:
           .word 0x0460                     // R7 channel enabled
           .word 0x1d010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch30:
           .word 0x0460                     // R7 channel enabled
           .word 0x1e010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

ch31:
           .word 0x0460                     // R7 channel enabled
           .word 0x1f010000 				         // R6 CPU Interrupt with prio 1, next run with prio lvl 3
           .word 0x0                            // R5
           .word 0x0 			                  // R4
           .word 0x0                            // R3
           .word 0x0                            // R2
           .word 0x0                            // R1
           .word 0x0                            // R0

//Page 4 of PMEM
.globl dptr
dptr:
           .word 0x0                            // PCP_FLAG0				offset 0	//Flags for Peterson's algorithm
           .word 0x0 			                // PCP_FLAG1				offset 1
           .word 0x0                            // PCP_TURN					offset 2

           .word 0x0							// PCP_PRIO					offset 3
           .word 0x0							// PCP_READY				offset 4
           .word 0x0							// PCP_PTASK				offset 5
           .word 0x0							// PCP_NEED_RESCHEDULE		offset 6
           .word 0x0							// Interrupt_Priority		offset 7
           .space 0x38							// fill up memory to reach new page in PMEM
//Page 5 and 6 of PMEM
		   .word 0x0
TASK_INFOS:
           .space 0x60, 0x0						// PCP_TTASK				offset 7	32 entries with 12 bytes per entry
