#ifndef OS_COM_RX_SIGNAL_TYPE_H
#define OS_COM_RX_SIGNAL_TYPE_H

#include "util/types.h"
#include "os/com/ComStack_Types.h"
#include "os/com/ascom/ConfigTypes.h"

namespace os { namespace com { namespace ascom {

class RxIpduType;

class RxSignalType : public RxSignalConfigType {
public:
	static void
	InitAll (RxSignalConfigType const * ConfigArray, RxSignalIdType NumRxSignals);

	static RxSignalType &
	Inst (RxSignalIdType SignalId);

	RxSignalIdType
	Id () const;

	// Called from upper layer to get data
	StatusType
	ReceiveSignal (void * SignalDataPtr);

	void
	Init (RxSignalConfigType const * ConfigPtr);

	void
	Init ();

	void
	Start (Bool Initialize);

	void
	Stop ();

	UInt
	BitLength () const;

	UInt8 *
	NewValue ();

	UInt8 const *
	NewValue () const;

	UInt8 *
	OldValue ();

	UInt8 const *
	OldValue () const;

	void
	RxIndication (UInt8 const * SduPtr);

private:
	static RxSignalType RxSignals_[cfOS_COM_ASCOM_RX_SIGNAL_MULTIPLICITY];
};

inline void
RxSignalType::InitAll (RxSignalConfigType const * ConfigArray, RxSignalIdType NumRxSignals) {
	RxSignalIdType i = 0;

	// We assume NumRxSignals <= cfOS_COM_ASCOM_RX_SIGNAL_MULTIPLICITY
	for (; i < NumRxSignals; ++i) {
		RxSignals_[i].Init(&ConfigArray[i]);
	}

	for (; i < cfOS_COM_ASCOM_RX_SIGNAL_MULTIPLICITY; ++i) {
		// FIXME Uninit or stop or similar
	}
}

inline RxSignalType &
RxSignalType::Inst (RxSignalIdType SignalId) {
	return RxSignals_[SignalId];
}

inline RxSignalIdType
RxSignalType::Id () const {
	return this - RxSignals_;
}

inline void
RxSignalType::Init (RxSignalConfigType const * ConfigPtr) {
	*(RxSignalConfigType*)this = *ConfigPtr;
	Init();
}

inline void
RxSignalType::Init () {
	for(RxByteIndexType i = 0; i < Length(); ++i) {
		GetBuffer<0> ()[i] = InitValue (i);
	}
}

inline void
RxSignalType::Start (Bool Initialize) {
	if(Initialize) {
		Init ();
	}
}

inline void
RxSignalType::Stop () {

}

inline UInt
RxSignalType::BitLength () const {
	// FIXME Check possible overflow of Int Res
	Int Res = MsByteIndex ();
	Res -= (Int)LsByteIndex ();
	Res = (Res < 0) ? -Res : Res;
	Res *= 8;
	return Res + 1 + MsBitIndex () - LsBitIndex ();
}

inline UInt8 *
RxSignalType::NewValue () {
	// In the default config the NewValue is directly forwarded into the OldValue
	// alias GetBuffer, so the data is directly accepted
	return GetBuffer<0> ();
}

inline UInt8 const *
RxSignalType::NewValue () const {
	// In the default config the NewValue is directly forwarded into the OldValue
	// alias GetBuffer, so the data is directly accepted
	return GetConstBuffer<0> ();
}

inline UInt8 *
RxSignalType::OldValue () {
	// In the default config the NewValue is directly forwarded into the OldValue
	// alias GetBuffer, so the data is directly accepted
	return GetBuffer<0> ();
}

inline UInt8 const *
RxSignalType::OldValue () const {
	// In the default config the NewValue is directly forwarded into the OldValue
	// alias GetBuffer, so the data is directly accepted
	return GetConstBuffer<0> ();
}

}}}

#endif
