#ifndef OS_COM_ASCOM_TX_IPDU_TYPE_H
#define OS_COM_ASCOM_TX_IPDU_TYPE_H

#include "string.h"
#include "os/com/ComStack_Types.h"
#include "os/com/ascom/ConfigTypes.h"

#define TX_IPDU_TYPE_MEMBER_UNSIGNED_MAXVAL(Member_, MAX_VAL) uint_t_value<MAX_VAL>::T Member_

namespace os { namespace com { namespace ascom {

class TxSignalType;
//class TxIpduGroupType;

class TxIpduType : public TxIpduConfigType {
public:
	static TxIpduType&
	Inst(TxIpduIdType TxIpduId);

	static void
	InitAll(TxIpduConfigType const * ConfigArray, TxIpduIdType TxIpduMultiplicity);

	static TxIpduIdType
	Multiplicity();

	TxIpduIdType
	Id() const;

	void
	Start(Bool Initialize);

	void
	Stop();

	void
	Init(TxIpduConfigType const * ConfigPtr);

	void
	Init();

	StatusType
	TxRequestDirect();

	UInt8 *
	SduBuffer();

	UInt8 const *
	SduBuffer() const;

	void
	TriggerTransmit (UInt8 * SduPtr);

	void
	TriggerIpduSend ();

	void
	TxConfirmation ();

private:
	StatusType
	TxRequest();

	// for connecting aspects
	StatusType
	TransmitEvent (PduInfoType const * PduInfoPtr);

	static TxIpduType TxIpdus_[cfOS_COM_ASCOM_TX_IPDU_MULTIPLICITY];
	static TxIpduIdType TxIpduMultiplicity_;
};

inline TxIpduIdType
TxIpduType::Id() const {
	return this - TxIpdus_;
}

inline TxIpduType&
TxIpduType::Inst(TxIpduIdType TxIpduId) {
	return TxIpdus_[TxIpduId];
}

inline void
TxIpduType::InitAll(TxIpduConfigType const * ConfigArray, TxIpduIdType TxIpduMultiplicity) {
	TxIpduIdType i = 0;

	// We assume TxIpduMultiplicity <= cfOS_COM_ASCOM_RX_SIGNAL_MULTIPLICITY
	for(; i < TxIpduMultiplicity; ++i) {
		TxIpdus_[i].Init(&ConfigArray[i]);
	}

	TxIpduMultiplicity_ = i;

	for(; i < cfOS_COM_ASCOM_TX_IPDU_MULTIPLICITY; ++i) {
		// FIXME Uninit or stop or similar
	}
}

inline TxIpduIdType
TxIpduType::Multiplicity() {
	return TxIpduMultiplicity_;
}

inline StatusType
TxIpduType::TxRequestDirect() {
	return COM_SERVICE_NOT_AVAILABLE;
}

inline UInt8 *
TxIpduType::SduBuffer() {
	return GetBuffer<0>();
}

inline UInt8 const *
TxIpduType::SduBuffer() const {
	return GetConstBuffer<0>();
}

inline void
TxIpduType::TriggerTransmit(UInt8 * SduPtr) {
	memcpy (SduPtr, SduBuffer(), Length ());
}

}}}

#endif
