// $Id: Alarm.h 1307 2010-11-17 12:36:18Z mike $
// Created: 2007-07-03 siwahofe

#ifndef __OS_KRN_ALARM_H__
#define __OS_KRN_ALARM_H__

#include "util/types.h"
#include "ciaoConfig.h"
#include "Task.h"
#include "os/krn/Counter.h"

namespace os { namespace krn {

struct Alarm {
	// data type to represent number of alarms
	typedef uint_t_value <(cfOS_NUMBER_OF_ALARMS)>::T Id;
	typedef UInt32 Ticks;

	// base
	Counter *configuration_;

	// state
	Bool armed_;
	Ticks absoluteTime_;
	Ticks cycleTime_;

	// action
	Task::Id task_; // task to activate / set event to (if applicable)

	static Alarm& Inst (Id alarmid);

	static Counter& getConfiguration (Id alarmid) {
		return *(Inst (alarmid).configuration_);
	}

	static void setArmed (Id alarmid, Bool armed) {
		Inst (alarmid).armed_ = armed;
	}
	static Bool getArmed (Id alarmid) {
		return Inst (alarmid).armed_;
	}

	static void setAbsoluteTime (Id alarmid, Ticks absoluteTime) {
		Inst (alarmid).absoluteTime_ = absoluteTime;
	}

	static void setRelativeTime (Id alarmid, Ticks relativeTime) {
		Alarm& a = Inst (alarmid);
		Ticks remaining = a.configuration_->maxallowedvalue-a.configuration_->value;

		if(remaining >= relativeTime) {
			a.absoluteTime_ = a.configuration_->value + relativeTime;
		} else {
			a.absoluteTime_ = relativeTime - remaining - 1;
		}
	}

	static Ticks getAbsoluteTime (Id alarmid) {
		return Inst (alarmid).absoluteTime_;
	}

	static void setCycleTime (Id alarmid, Ticks cycleTime) {
		Inst (alarmid).cycleTime_ = cycleTime;
	}
	static Ticks getCycleTime (Id alarmid) {
		return Inst (alarmid).cycleTime_;
	}

	static Task::Id getTask (Id alarmid) {
		return Inst (alarmid).task_;
	}

	static void trigger (Id alarmid);

	static void triggerAction (Id alarmid);

	static Ticks getRemainingTicks(Id alarmid) {
		Alarm& a = Inst(alarmid);
		if(a.absoluteTime_ > a.configuration_->value) {
			return a.absoluteTime_ - a.configuration_->value;
		} else {
			return a.absoluteTime_ + (a.configuration_->maxallowedvalue-a.configuration_->value);
		}
	}
};

extern Alarm theAlarms [];

inline Alarm& Alarm::Inst (Alarm::Id alarmid) {
    return theAlarms [alarmid];
}

}} // namespace os::krn

#endif // __OS_KRN_ALARM_H__
