#ifndef _OS_KRN_MESSAGEPORT_REVOKEWRITESENDER_H_
#define _OS_KRN_MESSAGEPORT_REVOKEWRITESENDER_H_

#include "os/krn/Messageport.h"
#include "os/mp/MPU.h"
#include "Context.h"

namespace os { namespace mp {

	template <typename MSGTYPE, int msgCount>
		class SenderPortWriteProtection {
			public:
				os::krn::SenderPort<MSGTYPE, msgCount> sp;

			inline MSGTYPE *allocateMessage() {
				MSGTYPE *msg = sp.allocateMessage();
				
				// grant access to message
				if(!os::mp::Context::isTrustedMPU()) {
					os::mp::MPU::enterTrusted();

					os::mp::Context::setACL(msg,msg+1);
					os::mp::Context::enableACL();
					os::mp::MPU::leaveTrusted_drv();
				}
				return msg;
			}

			inline void sendMessage(MSGTYPE *msg) {
				// revoke message access
				if(!os::mp::Context::isTrustedMPU()) {
					os::mp::MPU::enterTrusted();
					os::mp::Context::disableACL();
					os::mp::MPU::leaveTrusted_drv();
				}

				sp.sendMessage(msg);
			}
		};

}}; // namespace os::mp

#endif
