// $Id: sourcedoc.h 995 2009-08-11 14:26:31Z mike $
// Created:   26.05.2006 lohmann

/**
 * @file sourcedoc.h
 * @brief Contains macros and definitions for source code documentation purposes.
 *
 * @note This file is force-included into every translation unit. 
 */


#ifndef __SOURCEDOC_H__
#define __SOURCEDOC_H__

/**
 * @defgroup doc Documentation Macros
 * @brief Preprocessor macros that expand to nothing in C++ but should be used for documentation purposes.
 * 
 * All macros in this group expand to nothing. There only purpose is to document
 * attributes of classes, functions and other C++ elements. 
 * 
 * Currently two attributes are supported to document CiAO-specific semantics:
 * 
 * - CIAO_STD should be used to tag functionality that is considered to be 
 * part of the CiAO standard implementation. That is, the functionality that should be 
 * provided by hw::hal, os::dev::std and os::std on every plattform.
 * 
 * - CIAO_EXT should be used to tag functionality that is available on some plattforms only,
 * however, is still kind of common. This means that typically more than one actual plattform
 * implements this functionality. For instance, hw::hal::CPU::IRQL() is provided only on
 * plattforms that support interrupt levels by hardware.
 * 
 * All elements that are not tagged by CIAO_STD or CIAO_EXT have to be considered as plattform
 * or implementation-specific.
 * 
 * 
 * @{
 */ 


/**
 * @brief Provide a symbol to doxygen only
 * 
 * For doxygen this macro expands to x
 * while for the C++ compiler it expands to nothing. This
 * is quite useful to "name" anonymous C++ entities such 
 * as enums for documentation purposes
 */
#define DOXY_SYMBOL(x)




//! Functionality is part of the CiAO standard implementation (supported by all plattforms)
#define CIAO_STD
//! Functionality is supported by some CiAO plattforms only.
#define CIAO_EXT

/** @} */


#endif // __SOURCEDOC_H__

