#ifndef BITMANIP_TC_H
#define BITMANIP_TC_H

#include "util/metaprog.h"
#include "util/types.h"

namespace bitmanip {

namespace impl {

template <typename INT_TYPE>
struct twosComplementAlgo<INT_TYPE, true> {
	static INT_TYPE
	Do (register INT_TYPE Value) {
		// tc has two's complement arithmetic, so ...
		return -Value;
	}
};

template <typename INT_TYPE>
struct highest1BitAlgo<INT_TYPE, false, true> {
	static inline UInt
	Do (register INT_TYPE Value) {
		register UInt32 Tmp = Value;
		register UInt Res;
		asm("clz %0, %1" : "=d"(Res) : "d"(Tmp) );
		return 31 - Res;
	}
};

template <typename INT_TYPE>
struct highest0BitAlgo<INT_TYPE, false, true> {
	static inline UInt
	Do (register INT_TYPE Value) {
		register UInt32 Tmp = ~(UInt32)(INT_TYPE)~Value;
		register UInt Res;
		asm("clo %0, %1" : "=d"(Res) : "d"(Tmp) );
		return 31 - Res;
	}
};

template <typename INT_TYPE>
struct lowest1BitAlgo<INT_TYPE, false, true> {
	static inline UInt
	Do (register INT_TYPE Value) {
		return highest1BitAlgo<INT_TYPE, false>::Do(Value & twosComplementAlgo<UInt32>::Do(Value));
	}
};

template <typename INT_TYPE>
struct lowest0BitAlgo<INT_TYPE, false, true> {
	static inline UInt
	Do (register INT_TYPE Value) {
		return lowest1BitAlgo<UInt32, false>::Do (~Value);
	}
};

template<typename INT_TYPE>
struct testBitAlgo<INT_TYPE, false, true> {
	static inline Bool
	Do (register INT_TYPE Value, register UInt BitIndex) {
		register UInt32 Tmp = Value;
		register UInt32 TmpRes;
		// This impl is better than the generic impl, but ...
		// tricore-gcc still inserts a superfluous instruction to convert %d2 to a Bool which it already is.
		// Still no idea howto circumvent.
		asm ("extr.u %0, %1, %2, 1\n\t" : "=d"(TmpRes) : "d"(Tmp), "d"(BitIndex) );
		return TmpRes;
	}
};

template<typename INT_TYPE>
struct clearBitAlgo<INT_TYPE, false, true> {
	static inline INT_TYPE
	Do (register INT_TYPE Value, register UInt BitIndex) {
		register UInt32 Tmp = Value;
		asm ("insert %0, %1, 0, %2, 1\n\t" : "=d"(Tmp) : "d"(Tmp), "d"(BitIndex) );
		return (INT_TYPE) Tmp;
	}
};

template<typename INT_TYPE>
struct setBitAlgo<INT_TYPE, false, true> {
	static inline INT_TYPE
	Do (register INT_TYPE Value, register UInt BitIndex) {
		register UInt32 Tmp = Value;
		asm ("insert %0, %1, 1, %2, 1\n\t" : "=d"(Tmp) : "d"(Tmp), "d"(BitIndex) );
		return (INT_TYPE) Tmp;
	}
};

template<typename INT_TYPE>
struct extractAlgo<INT_TYPE, true> {
	static INT_TYPE
	Do (INT_TYPE Value, UInt BitIndex, UInt BitLength) {
		UInt64 PosWidth;
		UInt32 Res;
		asm ("mov %L1, %2\n\t"
			  "mov %H1, %3\n\t"
			  "extr.u %0, %4, %A1\n\t" : "=d"(Res), "=d"(PosWidth) : "d"(BitIndex), "d"(BitLength), "d"((UInt32)Value) );

		return (INT_TYPE)Res;
	}
};

} // namespace impl

} // namespace bitmanip

#endif

