#ifndef __Queue_h__
#define __Queue_h__
//! A Queue
/*! Container which allows efficent efficent access operations 
 * at the end and the beginning of the data. 
 */
#include "util/container/List.h"


template<class T,class C = Chain<T> >
class Queue :public List<T,C> {
	public:
		
		//! Type definition for base type
		typedef List<T,C> Base;
		
		//! Type definition for instatiated type
		typedef T value_t;
		
		//! Type definition for pointer to intantiated type
		typedef T* pointer_t;
		
		//! Type of the node which is managed by this container
		typedef typename Base::node_t node_t;
		
		//! Type of the iterator 
		typedef typename node_t::iterator_t iterator_t;
	
		//! Shorter name (STL conforming) of iterator type. 
		typedef iterator_t iterator;
	
	protected:
		//! Pointer to last queue element 
		node_t* _back;
	
	public:
		
		//! Constructor
		Queue():Base(),_back(0){}
		
		
		//! Constructor
		/*!
		 * \param elem will be the first element in the Queue
		 */ 
		Queue(node_t* item):Base(item){
			adjust_back();
		}
		
		//! Copy constructor
		Queue(Queue& queue):Base(queue.front()),_back(queue._back){}
		
		//! Constructor
		/*!
		 * When using this constructor the tail of the Queue will be 
		 * adjusted.
		 * 
		 * \param list is a refenrence to list object
		 * 
		 */
		Queue(Base& list):Base(list){
			adjust_back();
		}
		
		//! Add an element to the end of this Queue
		/*!
		 * \param value is the new element which should be queued
		 */
		void push_front(node_t* new_node){
			Base::push_front(new_node);
			if((this->_back) == 0){
				this->_back = new_node;
			}
		}
		
		
		//! Add a \ref node_t to the end of this Queue
		/*!
		 * \param value is the new element which should be queued
		 */
		void push_back(node_t* new_node){
			(this->_back)->setNext(new_node);
			if(this->_front == 0){
				this->_front = new_node;
			}
			this->_back=new_node;
		}
		
			
		//! Get the content last element of this Queue
		/*!
		 * \return Constant Reference to the value of last \ref node_t in this Queue
		 */
		node_t* back() const{
			return this->_back;
		}
		
		
		//! Adjust the \ref _back pointer to the last element 
		void adjust_back(){
			_back=Base::_front->findLast();
		}
		
		
		//! Get an iterator which marks the start of the list
		/*!
		 * \return an \ref iterator_t obeject instantiated with the last element
		 */
		iterator_t begin(){
			return this->_front;
		}
		
		//! Get an iterator which marks the start of the list
		/*!
		 * \return an \ref iterator_t obeject instantiated with the last element
		 */
		iterator_t last(){
			return this->_back;
		}
		
		//! Get an iterator which marks the end of the list
		/*!
		 * \return an \ref iterator_t object instantiated with the last element.
		 */
		iterator_t end(){
				return 0;
		}
};


#endif // __Queue_h__
