#ifndef __alloc_DlQueue_h__
#define __alloc_DlQueue_h__

#include "util/container/allocating/Queue.h"
#include "util/container/DlChain.h"


namespace allocating{
//! A double linked queue
/*! Container with constant insertion and deletion time.
 *  Has bidirectional iterator
 *  This container can be instantiated with custom types and/or 
 *  allocators
 */
template<class T, class A=DefaultAllocator >
class DlQueue : public Queue< T, A, DlChain<T> >{

public:
		
		typedef Queue< T, A, DlChain<T> > Base;
		
		typedef typename Base::node_t node_t;
		
		typedef typename Base::alloc_t alloc_t;
		
		//! Default constructor
		DlQueue():Base(){}
		
		//! Constructor
		/*!
		 * \param elem will be the first element in the Queue
		 */ 
		DlQueue(alloc_t& alloc):Base(alloc){}
		
		//! Copy Constructor 
		/*!
		 * \param queue is another queue object
		 */
		DlQueue(DlQueue& queue):Base(static_cast<Base>(queue)){}
		
		//! Generate a new list element and add it to the end of this List
		/*! 
		 * \param value is reference to value of the new element
		 */
		void pop_back(){
			if(this->_back != 0){
				node_t* tmp = this->_back;
				this->_back=(this->_back)->getPrev();
				Base::_alloc.free(static_cast<typename alloc_t::pointer_t >(tmp));
				if(this->_back == 0){
					this->_front = 0;
				}
			}
		}
		
}; //class Queue

}//namespace allocting


#endif // __alloc_DlQueue_h__


