#ifndef __NullAllocator_h__
#define __NullAllocator_h__

#include "util/types.h"
#include "util/defines.h"
#include "mem/BaseAllocator.h"

//! Allocator which can not allocate memory.
/**
 * This Allocator is used to instantiate Container which should allocate 
 * memory by them self.
 */

template<class T>
class NullAllocator : public BaseAllocator<T>{
public:
	//! Type definition for instatiated type
	typedef T value_t;
	//! Type definition for pointer to intantiated type
	typedef T* pointer_t;
	//! Type definition for this type of allocator
	typedef NullAllocator<T> alloc_t;
	
	template<class NT>
	struct rebind{
		typedef typename NullAllocator<NT>::alloc_t alloc_t;
	};   
	
	//! TODO
	pointer_t allocate(){
		STATIC_ERROR(this_allocator_does_not_implement_allocate);
		return 0;
	}
	
	//! TODO
	pointer_t allocate(int n){
		STATIC_ERROR(this_allocator_does_not_implement_allocate);
		return 0;
	}
	
	//! TODO
	void free (pointer_t elem){
		STATIC_ERROR(this_allocator_does_not_implement_free);
	}	
};



#endif //__NullAllocator_h__

