/** \author Christian Meier
 *  \version $Id$
 *
 *  This test checks whether the Development Error Checking
 *  is implemented correctly.
 */

#include "appcode.h"
#include "as/AS.h"
#include "objects.h"
#include "stdio.h"

#include "hw/dev/STM.h"
#include "can_Cfg.h"

#include "hw/dev/can/multican/CanDriverImpl.h"

UInt8 sduBuf[8] = "TEST!";
hw::dev::can::PduType myPdu0;

static hw::dev::can::DevelopmentErrorType ExpectedErrors[20] = {(hw::dev::can::DevelopmentErrorType) 0};
UInt ErrorIndex = 0;

volatile hw::dev::can::DevelopmentErrorType ActualErrors[20] = {(hw::dev::can::DevelopmentErrorType) 0};

static hw::dev::can::ReturnType ExpectedReturns[20] = {hw::dev::can::CAN_OK};
UInt ExpectedReturnIndex = 0;

static hw::dev::can::ReturnType ActualReturns[20] = {hw::dev::can::CAN_OK};
UInt ActualReturnIndex = 0;

void SmallDelay() {
	UInt32 TIMValFirst = hw::dev::STM::TIM<4>();
	while ((hw::dev::STM::TIM<4>() - TIMValFirst) > 10);
}

Bool OK = true;
static hw::dev::can::ControllerConfigType DummyControllerConfig;

void Alpha::functionTaskTask0(void) {
	ActualReturns[ActualReturnIndex++] =
		hw::dev::can::multican::CanDriverImpl::Inst().SetControllerMode(0, hw::dev::can::CAN_T_START);
	ExpectedReturns[ExpectedReturnIndex++] = hw::dev::can::CAN_OK;

	ActualReturns[ActualReturnIndex++] =
		hw::dev::can::multican::CanDriverImpl::Inst().SetControllerMode(1, hw::dev::can::CAN_T_START);
	ExpectedReturns[ExpectedReturnIndex++] = hw::dev::can::CAN_OK;

	ActualReturns[ActualReturnIndex++] =
		hw::dev::can::multican::CanDriverImpl::Inst().SetControllerMode(10, hw::dev::can::CAN_T_START);
	ExpectedReturns[ExpectedReturnIndex++] = hw::dev::can::CAN_NOT_OK;
	ExpectedErrors[ErrorIndex++] = hw::dev::can::CAN_E_PARAM_CONTROLLER;

	hw::dev::can::multican::CanDriverImpl::Inst().InitController(4, &DummyControllerConfig);
	ExpectedErrors[ErrorIndex++] = hw::dev::can::CAN_E_PARAM_CONTROLLER;

	hw::dev::can::multican::CanDriverImpl::Inst().InitController(0, 0);
	ExpectedErrors[ErrorIndex++] = hw::dev::can::CAN_E_PARAM_POINTER;

	hw::dev::can::multican::CanDriverImpl::Inst().DisableControllerInterrupts(5);
	ExpectedErrors[ErrorIndex++] = hw::dev::can::CAN_E_PARAM_CONTROLLER;

	hw::dev::can::multican::CanDriverImpl::Inst().EnableControllerInterrupts(5);
	ExpectedErrors[ErrorIndex++] = hw::dev::can::CAN_E_PARAM_CONTROLLER;

	ActualReturns[ActualReturnIndex++] =
		hw::dev::can::multican::CanDriverImpl::Inst().Write(0, 0);
	ExpectedReturns[ExpectedReturnIndex++] = hw::dev::can::CAN_NOT_OK;
	ExpectedErrors[ErrorIndex++] = hw::dev::can::CAN_E_PARAM_POINTER;

	myPdu0.Id_          = 22;
	myPdu0.Sdu_         = NULL;
	myPdu0.Length_      = 6;
	myPdu0.SwPduHandle_ = 2;

	ActualReturns[ActualReturnIndex++] =
		hw::dev::can::multican::CanDriverImpl::Inst().Write(0, &myPdu0);
	ExpectedReturns[ExpectedReturnIndex++] = hw::dev::can::CAN_NOT_OK;
	ExpectedErrors[ErrorIndex++] = hw::dev::can::CAN_E_PARAM_POINTER;

	myPdu0.Length_      = 9;
	myPdu0.Sdu_         = sduBuf;
	ActualReturns[ActualReturnIndex++] =
		hw::dev::can::multican::CanDriverImpl::Inst().Write(0, &myPdu0);
	ExpectedReturns[ExpectedReturnIndex++] = hw::dev::can::CAN_NOT_OK;
	ExpectedErrors[ErrorIndex++] = hw::dev::can::CAN_E_PARAM_DLC;

	myPdu0.Length_      = 6;
	ActualReturns[ActualReturnIndex++] =
		hw::dev::can::multican::CanDriverImpl::Inst().Write(0, &myPdu0);
	ExpectedReturns[ExpectedReturnIndex++] = hw::dev::can::CAN_OK;

	for(UInt i = 0; i < 20; ++i) {
		if(ActualErrors[i] != ExpectedErrors[i]) {
			printf("Error difference at index %d: Actual(%d), Expected(%d)\r\n", i, ActualErrors[i], ExpectedErrors[i]);
			OK = false;
		}
		if(ActualReturns[i] != ExpectedReturns[i]) {
			printf("Return value difference at index %d: Actual(%d), Expected(%d)\r\n", i, ActualReturns[i], ExpectedReturns[i]);
			OK = false;
		};
	}

	if(OK) {
		printf("Everything is OK!\r\n");
	} else {
		printf("Something went wrong!\r\n");
	}

	AS::TerminateTask();
}
