/** \author Christian Meier
 *  \version $Id$
 *
 *  This is a testcase for the tx and rx flags that have to be set
 *  internally when transmitted/received an LPDU. If everything is
 *  OK, the testcase should continually send and receive LPDUs.
 */

#include "appcode.h"
#include "as/AS.h"
#include "objects.h"
#include "stdio.h"

#include "canif_Cfg.h"

UInt8 sduBuf[8] = "FIRST!";

static os::com::PduInfoType PduInfo;
static UInt16 Round = 0;

void Alpha::functionTaskTask0(void) {
	AS::CanIf_SetControllerMode(0, hw::hal::can::CANIF_CS_STARTED);
	AS::CanIf_SetControllerMode(1, hw::hal::can::CANIF_CS_STARTED);

	while(true) {
		printf("Round %d: ", Round++);
		AS::CanIf_Transmit(CANIF_TX_LPDU_TESTDATA1, &PduInfo);

		// we wait until we got a tx confirmation
		while(AS::CanIf_ReadTxNotifStatus(CANIF_TX_LPDU_TESTDATA1) != hw::hal::can::CANIF_TX_RX_NOTIFICATION);

		printf("TX ");

		if(AS::CanIf_ReadTxNotifStatus(CANIF_TX_LPDU_TESTDATA1) != hw::hal::can::CANIF_NO_NOTIFICATION) {
			printf("\r\nError: Tx Confirmation flag is still set!\r\n");
		}

		// we wait until we got a rx indication
		while(AS::CanIf_ReadRxNotifStatus(CANIF_RX_LPDU_TESTDATA1) != hw::hal::can::CANIF_TX_RX_NOTIFICATION);

		printf("RX\r\n");

		if(AS::CanIf_ReadRxNotifStatus(CANIF_TX_LPDU_TESTDATA1) != hw::hal::can::CANIF_NO_NOTIFICATION) {
			printf("Error: Rx Indication flag is still set!\r\n");
		}
	}

	AS::CanIf_SetControllerMode(0, hw::hal::can::CANIF_CS_STOPPED);
	AS::CanIf_SetControllerMode(1, hw::hal::can::CANIF_CS_STOPPED);

	AS::TerminateTask();
}
