/*
 * Copyright (C) 2008 by egnite GmbH.
 * Copyright (C) 2001-2003 by egnite Software GmbH.
 * Copyright (c) 1993 by Digital Equipment Corporation.
 * Copyright (c) 1983, 1993 by The Regents of the University of California.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 *
 * -
 * Portions Copyright (C) 2000 David J. Hudson <dave@humbug.demon.co.uk>
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You can redistribute this file and/or modify it under the terms of the GNU
 * General Public License (GPL) as published by the Free Software Foundation;
 * either version 2 of the License, or (at your discretion) any later version.
 * See the accompanying file "copying-gpl.txt" for more details.
 *
 * As a special exception to the GPL, permission is granted for additional
 * uses of the text contained in this file.  See the accompanying file
 * "copying-liquorice.txt" for details.
 * -
 * Portions Copyright (c) 1983, 1993 by
 *  The Regents of the University of California.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * -
 * Portions Copyright (c) 1993 by Digital Equipment Corporation.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies, and that
 * the name of Digital Equipment Corporation not be used in advertising or
 * publicity pertaining to distribution of the document or software without
 * specific, written prior permission.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
 * CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/*
 * $Log$
 * Revision 1.3  2008/08/11 17:38:27  haraldkipp
 * Added an Ethernet protocol demultiplexer.
 *
 * Revision 1.2  2005/04/08 15:20:50  olereinhardt
 * added <sys/types.h> (__APPLE__) and <netinet/in.h> (__linux__)
 * for htons and simmilar.
 *
 * Revision 1.1.1.1  2003/05/09 14:41:27  haraldkipp
 * Initial using 3.2.1
 *
 * Revision 1.12  2003/02/04 18:14:57  harald
 * Version 3 released
 *
 * Revision 1.11  2002/06/26 17:29:35  harald
 * First pre-release with 2.4 stack
 *
 */

#include "message.h"
#include "icmp.h"
#include "hw/dev/LAN91.h"
#include "stdio.h"
#include "string.h"
#include "arp.h"
#include "in.h"

#define __byte_swap2(val)           \
    ((((val) & 0xff) << 8) |        \
     (((val) & 0xff00) >> 8))

#define __byte_swap4(val)           \
    ((((val) & 0xff) << 24) |       \
     (((val) & 0xff00) << 8) |      \
     (((val) & 0xff0000) >> 8) |    \
     (((val) & 0xff000000) >> 24))

#define htons(x) __byte_swap2(x)
#define htonl(x) __byte_swap4(x)
#define ntohs(x) __byte_swap2(x)
#define ntohl(x) __byte_swap4(x)

/*!
 * \brief Handle incoming ICMP packets.
 *
 * Incoming ICMP packets are processed in the background.
 * NutNet currently handles echo request and destination
 * unreachable packets. Any other packet type is silently 
 * discarded.
 *
 * \note This routine is called by the IP layer on incoming 
 *       ICMP datagrams. Applications typically do not call 
 *       this function.
 *
 * \param dev Identifies the device that received the packet.
 * \param nb  Pointer to a network buffer structure containing 
 *            the ICMP datagram.
 */
void os::net::Icmp::Input(Message& msg)
{
	if ( msg.size < sizeof(ICMPHDR) ) {
		return;
	}

	switch (msg.icmp.header.icmp_type) {
		case ICMP_ECHO:
			if (Reflect(ICMP_ECHOREPLY, msg)) {
				break;
			}
		case ICMP_UNREACH:
			Unreach(msg.icmp.header.icmp_code, msg);
	}
}

/*
 * Send out ICMP echo response.
 */
int os::net::Icmp::Reflect(uint8_t type, Message& msg)
{
	uint32_t dest = msg.ip.ip_src;

	msg.size=56;

	msg.ip.ip_src = htonl(hw::dev::NIC::getIP());
	msg.ip.ip_ttl = MAXTTL;

	return Output(type, dest, msg);
}

/*
 * Process incoming ICMP messages for destination unreachable.
 */
int os::net::Icmp::Unreach(int icmp_code, Message& msg)
{
    return 0;
}

/*!
 * \brief Send an ICMP datagram.
 *
 * Known ICMP types are:
 *
 * - #ICMP_ECHOREPLY Echo reply
 * - #ICMP_UNREACH Destination unreachable
 * - #ICMP_SOURCEQUENCH Packet lost
 * - #ICMP_REDIRECT Shorter route
 * - #ICMP_ECHO Echo reply
 * - #ICMP_ROUTERADVERT Router advertisement
 * - #ICMP_ROUTERSOLICIT Router solicitation
 * - #ICMP_TIMXCEED Time exceeded
 * - #ICMP_PARAMPROB Bad IP header
 * - #ICMP_TSTAMP Timestamp request
 * - #ICMP_TSTAMPREPLY Timestamp reply
 * - #ICMP_IREQ Information request
 * - #ICMP_IREQREPLY Information reply
 * - #ICMP_MASKREQ Address mask request
 * - #ICMP_MASKREPLY Address mask reply
 *
 * \param type Type of the ICMP message.
 * \param dest Destination IP address.
 * \param nb   Network buffer structure containing the datagram.
 *
 * \return 0 on success, -1 otherwise.
 */
int os::net::Icmp::Output(uint16_t type, uint32_t dest, Message& msg)
{
	msg.icmp.header.icmp_type = type;
	msg.icmp.header.icmp_cksum = 0;

	msg.icmp.header.icmp_cksum = os::net::Ip::Checksum(0, &msg.icmp, msg.size+sizeof(ICMPHDR));

	msg.size+= sizeof(ICMPHDR);

	return os::net::Ip::Output(IPPROTO_ICMP, dest, msg);
}

/*!
 * \brief Send an ICMP message to a given destination.
 *
 * \param type Type of the ICMP message. See NutIcmpOutput() for 
 *             a list of valid types.
 * \param code Type subcode.
 * \param spec Type specific data item.
 * \param dest IP address of the target.
 * \param nb   Network buffer structure containing the message to be sent.
 *             The structure must have been allocated by a previous
 *             call NutNetBufAlloc() and will be freed if this function
 *             returns with an error.
 *
 * \return 0 on success, -1 otherwise.
 */
int os::net::Icmp::Reply(uint16_t type, uint8_t code, uint32_t spec, uint32_t dest, Message& msg)
{
	msg.icmp.header.icmp_code = code;
	msg.icmp.header.icmp_spec = spec;

	return Output(type, dest, msg);
}

/*!
 * \brief Send an ICMP message as a response to a given destination.
 *
 * \param type Type of the ICMP message. See NutIcmpOutput() for 
 *             a list of valid types.
 * \param code Type subcode.
 * \param spec Type specific data item.
 * \param nb   Network buffer structure containing the previously recevied
 *             network packet. According to RFC792 the complete ip header 
 *             and the first 8 bytes of the transport netbuf is used as the
 *             application data for the response. If this function returns 
 *             with an error, the buffer is freed. The destination addess is
 *             taken from the ip header.
 *
 * \return 0 on success, -1 otherwise.
 */
int os::net::Icmp::Response(uint16_t type, uint8_t code, uint32_t spec, Message& msg)
{
	memcpy(&msg.icmp.data, &msg.ip, sizeof(IPHDR)+8);

	msg.size=sizeof(IPHDR)+8;

	return Reply(type, code, spec, msg.ip.ip_src, msg);
}


/*@}*/
