/** \author Christian Meier
 *  \version $Id$
 *
 */

#include "appcode.h"
#include "as/AS.h"
#include "util/types.h"
#include "objects.h"
#include "stdio.h"

#include "util/container/Bitset.h"

template <int BIT_LENGTH>
Bool
test_AllAndAny (Bitset<BIT_LENGTH>& Bs) {
	Bs.SetAll();
	if(Bs.AnyCleared()) return false;
	if(Bs.AllCleared()) return false;

	if(!Bs.AnySet()) return false;
	if(!Bs.AllSet()) return false;

	Bs.ClearAll();
	if(!Bs.AnyCleared()) return false;
	if(!Bs.AllCleared()) return false;

	if(Bs.AnySet()) return false;
	if(Bs.AllSet()) return false;

	return true;
}

template <int BIT_LENGTH>
Bool
test_SetClearTest (Bitset<BIT_LENGTH>& Bs) {
	Bs.ClearAll();
	for(unsigned int i = 0; i < BIT_LENGTH; ++i) {
		if(Bs.Test(i)) return false;
		Bs.Set(i);
		if(i > 0) {
			// Bit i-1 must not be set
			if(Bs.Test(i-1)) return false;
		}
		if(i < BIT_LENGTH - 1) {
			// Bit i+1 must not be set
			if(Bs.Test(i+1)) return false;
		}
		// Bit i must be set
		if(!Bs.Test(i)) return false;

		// Only Bit i is set, so ...
		if(Bs.Lowest1Bit() != i) return false;
		if(Bs.Highest1Bit() != i) return false;

		// Bit i is set
		if(!Bs.AnySet()) return false;
		if(Bs.AllCleared()) return false;

		Bs.Clear(i);

		if(i > 0) {
			// Bit i-1 must not be set
			if(Bs.Test(i-1)) return false;
		}
		if(i < BIT_LENGTH - 1) {
			// Bit i+1 must not be set
			if(Bs.Test(i+1)) return false;
		}
		// Bit i must be cleared
		if(Bs.Test(i)) return false;

		// All bits must be cleared
		if(!Bs.AllCleared()) return false;
	}

	// Special Test for first/last bit find methods

	return true;
}


template <int BIT_LENGTH>
bool
test_IndexIteratorOnes (Bitset<BIT_LENGTH>& Bs) {
	Bs.SetAll();
	typename Bitset<BIT_LENGTH>::BitIndexType FoundIndex;

	typename Bitset<BIT_LENGTH>::IndexIteratorOnes Iter = Bs.GetIndexIteratorOnes();

	for(unsigned int i = 0; i < BIT_LENGTH; i++) {
		if(!Iter.GetNext(FoundIndex)) return false;
		if(FoundIndex != i) return false;
	}

	if(Iter.GetNext(FoundIndex)) return false;

	for(unsigned j = 2; j < 5; ++j) {
		Bs.ClearAll();
		for(unsigned int i = 0; i < BIT_LENGTH; i+=j) {
			Bs.Set(i);
		}

		Iter = Bs.GetIndexIteratorOnes();
		for(unsigned int i = 0; i < BIT_LENGTH; i+=j) {
			if(!Iter.GetNext(FoundIndex)) return false;
			if(FoundIndex != i) return false;
		}

		if(Iter.GetNext(FoundIndex)) return false;
	}
	return true;
}


template <int BIT_LENGTH>
Bool
test_Bitset (Bitset<BIT_LENGTH>& Bs) {
	Bool ok = true;

	ok = ok && test_AllAndAny(Bs);
	ok = ok && test_SetClearTest(Bs);
	ok = ok && test_IndexIteratorOnes(Bs);

	return ok;
}

Bitset<1> Bs1;
Bitset<2> Bs2;
Bitset<4> Bs4;
Bitset<8> Bs8;
Bitset<16> Bs16;
Bitset<31> Bs31;
Bitset<32> Bs32;
Bitset<33> Bs33;
Bitset<48> Bs48;
Bitset<53> Bs53;
Bitset<63> Bs63;
Bitset<64> Bs64;
Bitset<96> Bs96;
Bitset<127> Bs127;
Bitset<128> Bs128;
Bitset<129> Bs129;
Bitset<1024> Bs1024;
//Bitset<65536> Bs65536;
//Bitset<65537> Bs65537;

TASKDEF(Alpha,Task0) {
	Bool ok = true;

	ok = ok && test_Bitset(Bs1);
	ok = ok && test_Bitset(Bs2);
	ok = ok && test_Bitset(Bs4);
	ok = ok && test_Bitset(Bs8);
	ok = ok && test_Bitset(Bs16);
	ok = ok && test_Bitset(Bs31);
	ok = ok && test_Bitset(Bs32);
	ok = ok && test_Bitset(Bs33);
	ok = ok && test_Bitset(Bs48);
	ok = ok && test_Bitset(Bs53);
	ok = ok && test_Bitset(Bs63);
	ok = ok && test_Bitset(Bs64);
	ok = ok && test_Bitset(Bs96);
	ok = ok && test_Bitset(Bs127);
	ok = ok && test_Bitset(Bs128);
	ok = ok && test_Bitset(Bs129);
	ok = ok && test_Bitset(Bs1024);
//	ok = ok && test_Bitset(Bs65536);
//	ok = ok && test_Bitset(Bs65537);

	if(ok) {
		printf("Test OK\n");
	} else {
		printf("Test Failed\n");
	}

	AS::TerminateTask();
}
