// very simple AVR test program: it simply tests if we are able to
// compile, link, and boot an AVR application.

#include "CiAO.h"

// these should not be necessary
#include "hw/dev/sensor/AVRADConverter.h"

class DS1621 {

    //! I2C address of the DS1621 chip
    enum { ADDRESS = 0x90 };

    //! I2C commands of the chip
    enum Command {
	START_CONVERT    = 0xee,
	STOP_CONVERT     = 0x22,
	READ_TEMPERATURE = 0xaa
    };

public:
    static bool temperature(Int8&, UInt8&);
    static bool start();
    static bool stop();
};

bool DS1621::temperature(Int8& t1,UInt8& t2){
    CiAO::I2C &i2c = CiAO::i2c ();
    Int8 temp1;
    UInt8 temp2;
    
    if (i2c.start(ADDRESS))
	if (i2c.write(READ_TEMPERATURE))
	    if (i2c.start(ADDRESS| 0x01))
		{
		    i2c.read((UInt8&)temp1,true);
		    i2c.read(temp2,false);
		    i2c.stop();
		    if (temp1>=0)
			t1=temp1;
		    else
			t1=temp1+((temp2) ? 1:0);
		    t2=((temp2) ? 5:0);    
		    return true;
	}
  return false;
}

bool DS1621::start(){
    CiAO::I2C &i2c = CiAO::i2c ();
    if (i2c.start(ADDRESS))
	if (i2c.write(START_CONVERT)){
	    i2c.stop();
	    return true;
	}
    return false;
}

bool DS1621::stop(){
    CiAO::I2C &i2c = CiAO::i2c ();
    if (i2c.start(ADDRESS))
	if (i2c.write(STOP_CONVERT)){
	    i2c.stop();
	    return true;
	}
    return false;
}

UInt32 wind; // better use CiAO::TimerCounter::Value

int main () {
    // initialize the CiAO system
    CiAO::start ();

    // get and initialize the display object
    CiAO::Display &lcd = CiAO::display ();
    lcd.init ();

    // start the temperature sensor
    DS1621::start ();

    // load timer and allow timer interrupts
    CiAO::Timer &timer = CiAO::timer ();
    timer.period (100000L); // 10ms
    timer.start ();

    // enable serial interface
    CiAO::Serial &serial = CiAO::serial ();
    serial.baudrate (CiAO::Serial::BAUDRATE_19200);
    serial.format (CiAO::Serial::EIGHT_DATABITS,
  	  	   CiAO::Serial::ONE_STOPBIT,
		   CiAO::Serial::NO_PARITY);
    serial.start ();
 
    asm("sei");

    // stetup PORTD
    PORTD |= 0x80;
    DDRD &= ~0x80;
    CiAO::TimerCounter::external (CiAO::TimerCounter::EXT_FALL);

    typedef ciao::AVRADConverter AD;
    AD::enable (true);
    while (true) {

	// serial message
	static int xx = 0;

	serial.send ('0' + xx);
	xx = (xx + 1) % 10;
	if (xx == 0)
	    serial.send ('\n');

	// wind
	lcd.setpos (0,0);
 	lcd.print ("Wind: ");
	lcd.putc ('0' + (wind/1000)%10);
	lcd.putc ('0' + (wind/100)%10);
	lcd.putc ('0' + (wind/10)%10);
	lcd.putc ('0' + (wind/1)%10);
	lcd.putc ('\n');
	
	// temperature
	Int8 t1;
	UInt8 t2;
	DS1621::temperature (t1, t2);
	lcd.print ("Temp: ");
	if (t1 < 0) {
	    lcd.print ("-");
	    t1 = -t1;
	}
	lcd.putc ('0' + (t1/10)%10);
	lcd.putc ('0' + (t1/1)%10);
	lcd.print (".");
	lcd.putc ('0' + (t2/1)%10);
	lcd.putc ('\n');

	// pressure
	AD::channel (0);
	AD::start ();
	while (!AD::ready ());
	UInt16 k = AD::value () - 768;
	int pres = 850 + (UInt8)((((k+(k<<1)+(k<<3)+(k<<4)+(k<<5))>>6)+(768*59/64)+143+1) - 850);
	lcd.print ("Pres: ");
	lcd.putc ('0' + (pres/1000)%10);
	lcd.putc ('0' + (pres/100)%10);
	lcd.putc ('0' + (pres/10)%10);
	lcd.putc ('0' + (pres/1)%10);
	for (long i = 0; i < 50000; i++);
    }
    AD::enable (false);
}

void ciao::AVRTimer1::tick () {
    wind = CiAO::TimerCounter::value ();
    CiAO::TimerCounter::value (0);
}
