<xsl:stylesheet version='1.0'
xmlns:xsl='http://www.w3.org/1999/XSL/Transform'>
  <xsl:output method="xml" indent="yes"/>
  <xsl:strip-space elements="*" />
  <xsl:template match="/">
    <ComStacks xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xsi:noNamespaceSchemaLocation="./ComStacks.xsd">
      <xsl:apply-templates />
    </ComStacks>
  </xsl:template>
  <xsl:template match="ComInfrastructure/Node">
    <Node>
      <xsl:attribute name="Name">
        <xsl:value-of select="@Name" />
      </xsl:attribute>
      <Can>
        <xsl:for-each select="CanModule">
          <xsl:call-template name="CanModule">
            <xsl:with-param name="CurrentCanModule" select="." />
          </xsl:call-template>
        </xsl:for-each>
	  </Can>
	  <CanIf>
		<xsl:for-each select="CanModule/Plug">
			<LogicalController>
				<xsl:attribute name="Id"><xsl:value-of select="position()-1"/></xsl:attribute>
				<xsl:attribute name="ModuleRef"><xsl:value-of select="../@Name"/></xsl:attribute>
				<xsl:attribute name="ControllerRef"><xsl:value-of select="@ControllerId"/></xsl:attribute>
			</LogicalController>
		</xsl:for-each>
        <xsl:call-template name="CreateCanLPdusPerNode">
          <xsl:with-param name="CurrentNodeName" select="@Name" />
        </xsl:call-template>
      </CanIf>
    </Node>
  </xsl:template>
  <xsl:key name="PlugKey" match="//Node/*/Plug" use="@Id" />
  <xsl:key name="ConnectKey" match="//Connect" use="@Plug" />
  <xsl:decimal-format name="NanTo0" NaN="0" />
  <xsl:template name="CanModule">
    <xsl:param name="CurrentCanModule" />
    <Module>
      <xsl:attribute name="Name">
        <xsl:value-of select="@Name" />
      </xsl:attribute>
<!--      <xsl:attribute name="TimeoutDurationFactor">
        <xsl:value-of select="@TimeoutDurationFactor" />
      </xsl:attribute>
-->
      <!-- First we create the Controller configs -->
      <xsl:for-each select="Plug[key('ConnectKey', @Id)]">
        <!-- For every Plug that has a respective Connect statement -->
        <xsl:sort select="@ControllerId" order="ascending"
        data-type="number" />
        <Controller>
		  <xsl:attribute name="ControllerId">
			<xsl:value-of select="@ControllerId"/>
		  </xsl:attribute>
		  <xsl:element name="Config">
			  <xsl:attribute name="ConfigId">0</xsl:attribute>
			  <xsl:attribute name="BaudRate"><xsl:value-of select="key('ConnectKey', @Id)/../@BaudRate" /></xsl:attribute>
			  <xsl:copy-of select="key('PlugKey', @Id)/@PropSeg | key('PlugKey', @Id)/@PhaseSeg1
			  					   | key('PlugKey', @Id)/@PhaseSeg2" />
			  <xsl:attribute name="TimeQuanta"><xsl:value-of select="1 div (key('ConnectKey', @Id)/../@BaudRate * ( key('PlugKey', @Id)/@PropSeg 
			  			  																					        + key('PlugKey', @Id)/@PhaseSeg1
			  					   																			        + key('PlugKey', @Id)/@PhaseSeg2
			  					   																			        + 1 ))" /> </xsl:attribute>
		  </xsl:element>
        </Controller>
      </xsl:for-each>
      <!-- Then we create the hardware objects -->
      <xsl:variable name="SinkSourceSet"
      select="//CanNetwork/CanFrame/Sink[key('PlugKey', @Plug)[../../@Name = $CurrentCanModule/../@Name and ../@Name = $CurrentCanModule/@Name]] | //CanNetwork/CanFrame/Source[key('PlugKey', @Plug)[../../@Name = $CurrentCanModule/../@Name and ../@Name = $CurrentCanModule/@Name]]" />
      <xsl:for-each select="$SinkSourceSet">
        <xsl:sort select="name()" order="descending"
        data-type="text" />
        <xsl:sort select="format-number(floor(../ExtendedCanId/@Id div 262144), '#', 'NanTo0') + format-number(../StandardCanId/@Id, '#', 'NanTo0')"
        order="ascending" data-type="number" />
        <xsl:sort select="concat(name(../StsandardCanId), name(../ExtendedCanId))"
        order="descending" data-type="text" />
        <xsl:choose>
          <xsl:when test="name(.)='Sink'">
            <xsl:element name="RxHwObject">
              <xsl:attribute name="ObjId">
                <xsl:number level="any" value="position() - 1" />
              </xsl:attribute>
              <xsl:attribute name="ControllerRef">
                <xsl:value-of select="$CurrentCanModule/Plug/@ControllerId[../@Id = current()/@Plug]" />
              </xsl:attribute>
              <xsl:attribute name="HwObjectDummyId">
                <xsl:value-of select="generate-id(.)" />
              </xsl:attribute>
              <xsl:choose>
                <xsl:when test="count(../StandardCanId) &gt; 0">
                  <xsl:element name="StandardCanHwFilter">
                    <xsl:attribute name="Id">
                      <xsl:value-of select="../StandardCanId/@Id" />
                    </xsl:attribute>
                    <xsl:attribute name="Mask">2047</xsl:attribute>
                  </xsl:element>
                </xsl:when>
                <xsl:when test="count(../ExtendedCanId) &gt; 0">
                  <xsl:element name="ExtendedCanHwFilter">
                    <xsl:attribute name="Id">
                      <xsl:value-of select="../ExtendedCanId/@Id" />
                    </xsl:attribute>
                    <xsl:attribute name="Mask">536870911</xsl:attribute>
                  </xsl:element>
                </xsl:when>
                <xsl:otherwise></xsl:otherwise>
              </xsl:choose>
              <!--              <xsl:copy-of select="../StandardCanId | ../ExtendedCanId" /> -->
            </xsl:element>
          </xsl:when>
          <xsl:otherwise>
            <xsl:element name="TxHwObject">
              <xsl:attribute name="ObjId">
                <xsl:number level="any" value="position() - 1" />
              </xsl:attribute>
              <xsl:attribute name="ControllerRef">
                <xsl:value-of select="$CurrentCanModule/Plug/@ControllerId[../@Id = current()/@Plug]" />
              </xsl:attribute>
              <xsl:attribute name="HwObjectDummyId">
                <xsl:value-of select="generate-id(.)" />
              </xsl:attribute>
              <xsl:copy-of select="../StandardCanId | ../ExtendedCanId" />
            </xsl:element>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
    </Module>
  </xsl:template>
  <xsl:template name="CreateCanLPdusPerNode">
    <xsl:param name="CurrentNodeName" />
    <xsl:for-each select="//CanNetwork/CanFrame/Sink[key('PlugKey', @Plug)[../../@Name = $CurrentNodeName]]">

      <xsl:sort select="name()" order="descending"
      data-type="text" />
      <xsl:sort select="format-number(floor(../ExtendedCanId/@Id div 262144), '#', 'NanTo0') + format-number(../StandardCanId/@Id, '#', 'NanTo0')"
      order="ascending" data-type="number" />
      <xsl:sort select="concat(name(../StandardCanId), name(../ExtendedCanId))"
      order="descending" data-type="text" />
          <RxLpdu>
            <xsl:attribute name="LPduId">
              <xsl:number level="any" value="position() - 1" />
            </xsl:attribute>
            <xsl:attribute name="Name">
              <xsl:value-of select="../@Name" />
            </xsl:attribute>
            <xsl:attribute name="HwObjectDummyRef">
              <xsl:value-of select="generate-id(.)" />
            </xsl:attribute>
			<xsl:attribute name="ModuleRef">
				<xsl:value-of select="key('PlugKey',@Plug)/../@Name"/>
			</xsl:attribute>
            <xsl:copy-of select="../StandardCanId | ../ExtendedCanId" />
          </RxLpdu>
    </xsl:for-each>

    <xsl:for-each select="//CanNetwork/CanFrame/Source[key('PlugKey', @Plug)[../../@Name = $CurrentNodeName]]">

      <xsl:sort select="name()" order="descending"
      data-type="text" />
      <xsl:sort select="format-number(floor(../ExtendedCanId/@Id div 262144), '#', 'NanTo0') + format-number(../StandardCanId/@Id, '#', 'NanTo0')"
      order="ascending" data-type="number" />
      <xsl:sort select="concat(name(../StsandardCanId), name(../ExtendedCanId))"
      order="descending" data-type="text" />
          <TxLpdu>
            <xsl:attribute name="LPduId">
              <xsl:number level="any" value="position() - 1" />
            </xsl:attribute>
            <xsl:attribute name="Name">
              <xsl:value-of select="../@Name" />
            </xsl:attribute>
            <xsl:attribute name="HwObjectDummyRef">
              <xsl:value-of select="generate-id(.)" />
            </xsl:attribute>
			<xsl:attribute name="ModuleRef">
				<xsl:value-of select="key('PlugKey',@Plug)/../@Name"/>
			</xsl:attribute>
            <xsl:copy-of select="../StandardCanId | ../ExtendedCanId" />
          </TxLpdu>
    </xsl:for-each>
  </xsl:template>
</xsl:stylesheet>
