#!/usr/bin/perl

use warnings;
use strict;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.ah$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <bind.ah>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
open (SRC, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";


print SRC <<EOH;
// generated CiAO runtime
#include "as/autosar_pointcuts.h"

// usercode and hook routine includes
EOH
print SRC "#include \"$_\"\n" foreach $cfg->getAppHeaders();
print SRC "#include \"$_\"\n" foreach $cfg->getHookHeaders();

print SRC <<EOH;

aspect Bindings {

	// ISR bindings
EOH
foreach my $isr ($cfg->getISRs()) {
	my $isrsrc = $isr->interruptSource();
	my $fn = $isr->entryFunction();

	print SRC <<EOISR;
	advice execution ( " % hw::irq::${isrsrc}::Handler (...)" ) : before () {
		$fn();
	}
EOISR
}

print SRC "\n// HOOK bindings\n";

for my $hook (qw(PreTaskHook PostTaskHook StartupHook ShutdownHook ErrorHook)) {
	my $hf = $cfg->hookFunction(lc($hook));
	next unless $hf;

	print SRC <<EOHOOK;
	advice execution(asInternal$hook()) : before() {
		$hf();
	}
EOHOOK
}

print SRC "};\n";

close(SRC);
