#!/usr/bin/perl

use warnings;
use strict;

use SystemConfiguration;
use Data::Dumper;

sub app_code {
	my $app = shift();
	my $retVal;

	$retVal .= <<EOH;
  __CIAO_$app->{name}CODE_BEGIN = . ;
  .text.$app->{name} : {
EOH

	foreach ( keys % {$app->{componentref}} ) {
		my $len = length($_);
		$retVal .= <<EOH;
    *(.text._ZN$len$_*)
EOH
	}

	$retVal .= <<EOH;
    . = ALIGN(4);
  } > ext_cram
  __CIAO_$app->{name}CODE_END = . ;
EOH

	return $retVal;
}

sub app_data {
	my $app = shift();
	my $retVal = "";

	$retVal .=<<EOH;
  __CIAO_$app->{name}STACKS_BEGIN = . ;
EOH
	foreach my $c ( keys %{$app->{componentref}} ) {
		foreach my $t ( keys %{$app->{componentref}->{$c}->{taskref}} ) {
			my $len = length($t) + length('stackE');
			$retVal .=<<EOH
  __CIAO_${t}STACK_BEGIN = . ;
  .bss.stack.${t}	(NOLOAD) : {
    . = . + BUFFER_INTERSTACK;
    *(.bss._ZN2os3krn$len${t}_stackE)
  } > ext_dram = 0
  __CIAO_${t}STACK_END = . ;
EOH
		}
	}
	$retVal .=<<EOH;
  __CIAO_$app->{name}STACKS_END = . ;
EOH

	$retVal .=<<EOH;
  __CIAO_$app->{name}DATA_BEGIN = . ;
  .data.$app->{name} : {
    . = . + BUFFER_INTERAPP;
    *(.data.__CIAO_APPDATA_$app->{name}_*)
EOH

	foreach my $c ( keys %{$app->{componentref}} ) {
		my $len = length($c);
		$retVal .=<<EOH;
    *(.data._ZN$len$c*)
    *(.data._ZZN$len$c*)
EOH
	}

	$retVal .=<<EOH;
    . = ALIGN(4);
  } > ext_dram AT> ext_cram
  .bss.$app->{name}	(NOLOAD) : {
    *(.bss.__CIAO_APPDATA_$app->{name}_*)
EOH

	foreach my $c ( keys %{$app->{componentref}} ) {
		my $len = length($c);
		$retVal .=<<EOH;
    *(.bss._ZN$len$c*)
    *(.bss._ZZN$len$c*)
EOH
	}

	$retVal .=<<EOH;
    . = ALIGN(4);
  } > ext_dram = 0
  __CIAO_$app->{name}DATA_END = . ;
EOH
	return $retVal;
}

sub app_data_untrusted {}
sub app_data_trusted {}

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.x$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <ldciao.x>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
open (SRC, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";

print SRC <<EOH;
/* generated CiAO runtime */

SECTIONS {
  BUFFER_TNT = 7;
  BUFFER_INTERAPP = 7;
  BUFFER_INTERSTACK = 7;

  .startup : {
  	KEEP (*(.startup_code))
  	. = ALIGN(8);
  } > ext_cram

  __CIAO_CODE_BEGIN = . ;
  __CIAO_NTCODE_BEGIN = . ;
EOH

foreach ( $cfg->getApplications() ) {
	if ( $_->{trusted} eq 'false' ) {
		print SRC app_code($_);
	}
}

print SRC <<EOH;
  __CIAO_NTCODE_END = . ;
  __CIAO_TCODE_BEGIN = . ;
EOH

foreach ( $cfg->getApplications() ) {
	if ( $_->{trusted} eq 'true' ) {
		print SRC app_code($_);
	}
}

print SRC <<EOH;
  __CIAO_OSCODE_BEGIN = . ;
  .text.os : {
    *(.text._ZN2os*)
    . = ALIGN(4);
  } > ext_cram
  __CIAO_OSCODE_END = . ;
  __CIAO_TCODE_END = . ;
  /* dummy section, to move the location counter into the data region
     without an explicit and ugly assignment */
  .dummy_start_data (NOLOAD) : {
   __CIAO_DATA_BEGIN = . ;
 __CIAO_NTDATA_BEGIN = . ;
  } > ext_dram
  .my_ctors : {
    __CTORS_START = . ;
    KEEP (*(.ctors*))
    __CTORS_END = . ;
  } > ext_dram
EOH

foreach ( $cfg->getApplications() ) {
	if ( $_->{trusted} eq 'false' ) {
		print SRC app_data($_);
	}
}

print SRC <<EOH;
  __CIAO_NTDATA_END = . ;
  .tnt_buffer	(NOLOAD) : {
  . = . + BUFFER_TNT;
  . = ALIGN(4);
  } > ext_dram = 0
  __CIAO_TDATA_BEGIN = . ;
EOH

foreach ( $cfg->getApplications() ) {
	if ( $_->{trusted} eq 'true' ) {
		print SRC app_data($_);
	}
}

print SRC <<EOH;
  __CIAO_OSDATA_BEGIN = . ;
  .data.os : {
    . = ALIGN(4);
    *(.data._ZN2os*)
  } > ext_dram AT> ext_cram
  .bss.os 	(NOLOAD) : {
    . = ALIGN(4);
    *(.bss._ZN2os*)
  } > ext_dram
  __CIAO_OSDATA_END = . ;
  __CIAO_TDATA_END = . ;
EOH

print SRC <<EOH;
  .copy_sec : {
    . = ALIGN(4);
    PROVIDE(__clear_table = .);
    LONG(0 + ADDR(.bss));          LONG(SIZEOF(.bss));
    LONG(0 + ADDR(.sbss));         LONG(SIZEOF(.sbss));
    LONG(0 + ADDR(.zbss));         LONG(SIZEOF(.zbss));
    LONG(__PCP_CODE_RAM_BEGIN);    LONG(__PCP_CODE_RAM_SIZE);
EOH

foreach my $app ( $cfg->getApplications() ) {
	print SRC <<EOH;
    LONG(0 + ADDR(.bss.$app->{name}));    LONG(SIZEOF(.bss.$app->{name}));
EOH
}

print SRC <<EOH;
    LONG(0 + ADDR(.bss.os));    LONG(SIZEOF(.bss.os));
    LONG(-1);    LONG(-1);

    PROVIDE(__copy_table = .);
    LONG(LOADADDR(.data));    LONG(0 + ADDR(.data));    LONG(SIZEOF(.data));
    LONG(LOADADDR(.sdata));   LONG(0 + ADDR(.sdata));   LONG(SIZEOF(.sdata));
    LONG(LOADADDR(.zdata));   LONG(0 + ADDR(.zdata));   LONG(SIZEOF(.zdata));
    LONG(LOADADDR(.pcpdata)); LONG(0 + ADDR(.pcpdata)); LONG(SIZEOF(.pcpdata));
    LONG(LOADADDR(.pcptext)); LONG(0 + ADDR(.pcptext)); LONG(SIZEOF(.pcptext));
EOH

foreach my $app ( $cfg->getApplications() ) {
	print SRC <<EOH;
    LONG(LOADADDR(.data.$app->{name}));    LONG(0 + ADDR(.data.$app->{name})); LONG(SIZEOF(.data.$app->{name}));
EOH
}

print SRC <<EOH;
    LONG(LOADADDR(.data.os));    LONG(0 + ADDR(.data.os)); LONG(SIZEOF(.data.os));
    LONG(-1);                 LONG(-1);                  LONG(-1);
    . = ALIGN(8);
  } > ext_cram
}
EOH

close(SRC);

