<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet 
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
  version="1.0">

  <xsl:include href="../common.xsl" />
  
  <xsl:template match="ciaoApp">
	<xsl:text>/* generated CiAO runtime */&#10;&#10;</xsl:text>
	<xsl:text>SECTIONS {&#10;</xsl:text>
    <xsl:text>  BUFFER_TNT = 7;&#10;</xsl:text>
    <xsl:text>  BUFFER_INTERAPP = 7;&#10;</xsl:text>
    <xsl:text>  BUFFER_INTERSTACK = 7;&#10;</xsl:text>

	<xsl:text>  .startup : {&#10;</xsl:text>
	<xsl:text>  	KEEP (*(.startup_code))&#10;</xsl:text>
	<xsl:text>  	. = ALIGN(8);&#10;</xsl:text>
	<xsl:text>  } > ext_cram = 0&#10;</xsl:text>
	
	<xsl:text>  __CIAO_CODE_BEGIN = . ;&#10;</xsl:text>
	<xsl:text>  __CIAO_NTCODE_BEGIN = . ;&#10;</xsl:text>

	<xsl:call-template name="app_code">
  		<xsl:with-param name="apps" select="osapplication[@trusted='false']"/>
  	</xsl:call-template>

	<xsl:text>  __CIAO_NTCODE_END = . ;&#10;</xsl:text>
	<xsl:text>  __CIAO_TCODE_BEGIN = . ;&#10;</xsl:text>

	<xsl:call-template name="app_code">
  		<xsl:with-param name="apps" select="osapplication[@trusted='true']"/>
  	</xsl:call-template>

	<xsl:text>  __CIAO_OSCODE_BEGIN = . ;&#10;</xsl:text>
	<xsl:text>  .text.os : {&#10;</xsl:text>
	<xsl:text>    *(.text._ZN2os*)&#10;</xsl:text>
	<xsl:text>  } > ext_cram&#10;</xsl:text>
	<xsl:text>  __CIAO_OSCODE_END = . ;&#10;</xsl:text>
	<xsl:text>  __CIAO_TCODE_END = . ;&#10;</xsl:text>
	<xsl:text>  /* dummy section, to move the location counter into the data region&#10;</xsl:text>	
	<xsl:text>     without an explicit and ugly assignment */&#10;</xsl:text>	
	<xsl:text>  .dummy_start_data (NOLOAD) : {&#10;</xsl:text>
	<xsl:text>   __CIAO_DATA_BEGIN = . ;&#10;</xsl:text>
	<xsl:text>   __CIAO_NTDATA_BEGIN = . ;&#10;</xsl:text>
	<xsl:text>  } > ext_dram&#10;</xsl:text>
	
	<xsl:call-template name="app_data">
  		<xsl:with-param name="apps" select="osapplication[@trusted='false']"/>
  	</xsl:call-template>

	<xsl:text>  __CIAO_NTDATA_END = . ;&#10;</xsl:text>
	<xsl:text>  .tnt_buffer	(NOLOAD) : {&#10;</xsl:text>  
	<xsl:text>  	. = . + BUFFER_TNT;&#10;</xsl:text>  
	<xsl:text>  } > ext_dram = 0&#10;</xsl:text>  
	<xsl:text>  __CIAO_TDATA_BEGIN = . ;&#10;</xsl:text>

	<xsl:call-template name="app_data">
  		<xsl:with-param name="apps" select="osapplication[@trusted='true']"/>
  	</xsl:call-template>

	<xsl:text>  __CIAO_OSDATA_BEGIN = . ;&#10;</xsl:text>
	<xsl:text>  .data.os : {&#10;</xsl:text>
	<xsl:text>    *(.data._ZN2os*)&#10;</xsl:text>
	<xsl:text>  } > ext_dram&#10;</xsl:text>
	<xsl:text>  .bss.os 	(NOLOAD) : {&#10;</xsl:text>
	<xsl:text>    *(.bss._ZN2os*)&#10;</xsl:text>
	<xsl:text>  } > ext_dram&#10;</xsl:text>
	<xsl:text>  __CIAO_OSDATA_END = . ;&#10;</xsl:text>
	<xsl:text>  __CIAO_TDATA_END = . ;&#10;</xsl:text>

	<xsl:text>}&#10;</xsl:text>
  </xsl:template>

  <xsl:template name="app_code">
  	<xsl:param name="apps"/>
  	<xsl:for-each select="$apps">
		<xsl:text>  __CIAO_</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>CODE_BEGIN = . ;&#10;</xsl:text>
		<xsl:text>  .text.</xsl:text>
                <xsl:value-of select="@name"/>
                <xsl:text> : {&#10;</xsl:text>
                
		<xsl:for-each select="componentref">
			<xsl:text>    *(</xsl:text>
			<xsl:text>.text.</xsl:text>
			<xsl:text>_ZN</xsl:text>
			<xsl:variable name="to_mangle">
				<xsl:call-template name="component_classname">
					<xsl:with-param name="name" select="@name"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:call-template name="namespace_mangle">
				<xsl:with-param name="namespace" select="$to_mangle"/>
			</xsl:call-template>			
			<xsl:text>*)&#10;</xsl:text>
		</xsl:for-each>
		
		<xsl:text>  } > ext_cram&#10;</xsl:text>
		<xsl:text>  __CIAO_</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>CODE_END = . ;&#10;</xsl:text>		
	</xsl:for-each>
  </xsl:template>

  <xsl:template name="app_data">
  	<xsl:param name="apps"/>
  	<xsl:for-each select="$apps">

		<!-- Task Stacks -->	
		<xsl:text>  __CIAO_</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>STACKS_BEGIN = . ;&#10;</xsl:text>

		<xsl:for-each select="componentref">
		<xsl:variable name="nom" select="@name"/>
		<xsl:for-each select="/ciaoApp/component[@name=$nom]">
			
		<xsl:for-each select="taskref">
			<xsl:variable name="to_mangle">
				<xsl:call-template name="task_funcname">
					<xsl:with-param name="name" select="@name"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:variable name="common_remainder">
				<xsl:text>_ZZN</xsl:text>
				<xsl:call-template name="namespace_mangle">
					<xsl:with-param name="namespace" select="$to_mangle"/>
				</xsl:call-template>			
				<xsl:text>Ev*)&#10;</xsl:text>
				<xsl:text>  } > ext_dram&#10;</xsl:text>		
			</xsl:variable>		
			
			<xsl:text>  __CIAO_</xsl:text>
			<xsl:value-of select="@name"/>
			<xsl:text>STACK_BEGIN = . ;&#10;</xsl:text>
			<xsl:text>  .bss.stack.</xsl:text>
	        <xsl:value-of select="@name"/>
    	    <xsl:text>	(NOLOAD) : {&#10;</xsl:text>                		
			<xsl:text>  . = . + BUFFER_INTERSTACK;&#10;</xsl:text>
			<xsl:text>    *(</xsl:text>
			<xsl:text>.bss._ZN2os3krn</xsl:text>
			<xsl:value-of select="string-length(@name)+6"/>						
			<xsl:value-of select="@name"/>			
			<xsl:text>_stackE)&#10;</xsl:text>		
			<xsl:text>  } > ext_dram = 0&#10;</xsl:text>					
			<xsl:text>  __CIAO_</xsl:text>
			<xsl:value-of select="@name"/>
			<xsl:text>STACK_END = . ;&#10;</xsl:text>			
		</xsl:for-each>

		</xsl:for-each>
		</xsl:for-each>

		<xsl:text>  __CIAO_</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>STACKS_END = . ;&#10;</xsl:text>
		<!-- Task Stacks end -->

		<xsl:text>  __CIAO_</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>DATA_BEGIN = . ;&#10;</xsl:text>

		<xsl:text>  .data.</xsl:text>
	        <xsl:value-of select="@name"/>
    	    <xsl:text> : {&#10;</xsl:text>                

			<xsl:if test="position()>1">
				<xsl:text>  . = . + BUFFER_INTERAPP;&#10;</xsl:text>
			</xsl:if>

		<xsl:text>    *(.data.__CIAO_APPDATA_</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>_*)&#10;</xsl:text>

		<xsl:for-each select="componentref">
			<xsl:text>    *(</xsl:text>
			<xsl:text>.data.</xsl:text>
			<xsl:text>_ZN</xsl:text>
			<xsl:variable name="to_mangle">
				<xsl:call-template name="component_classname">
					<xsl:with-param name="name" select="@name"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:call-template name="namespace_mangle">
				<xsl:with-param name="namespace" select="$to_mangle"/>
			</xsl:call-template>			
			<xsl:text>*)&#10;</xsl:text>

			<xsl:text>    *(</xsl:text>
			<xsl:text>.data.</xsl:text>
			<xsl:text>_ZZN</xsl:text>
			<xsl:call-template name="namespace_mangle">
				<xsl:with-param name="namespace" select="$to_mangle"/>
			</xsl:call-template>			
			<xsl:text>*)&#10;</xsl:text>

		</xsl:for-each>
		<xsl:text>  } > ext_dram = 0&#10;</xsl:text>

		<!-- BSS start -->
		<xsl:text>  .bss.</xsl:text>
	        <xsl:value-of select="@name"/>
    	    <xsl:text>	(NOLOAD) : {&#10;</xsl:text>                
			
		<xsl:text>    *(.bss.__CIAO_APPDATA_</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>_*)&#10;</xsl:text>

		<!-- component bss -->
		<xsl:for-each select="componentref">
			<xsl:text>    *(</xsl:text>
			<xsl:text>.bss.</xsl:text>
			<xsl:text>_ZN</xsl:text>
			<xsl:variable name="to_mangle">
				<xsl:call-template name="component_classname">
					<xsl:with-param name="name" select="@name"/>
				</xsl:call-template>
			</xsl:variable>
			<xsl:call-template name="namespace_mangle">
				<xsl:with-param name="namespace" select="$to_mangle"/>
			</xsl:call-template>			
			<xsl:text>*)&#10;</xsl:text>

			<xsl:text>    *(</xsl:text>
			<xsl:text>.bss.</xsl:text>
			<xsl:text>_ZZN</xsl:text>
			<xsl:call-template name="namespace_mangle">
				<xsl:with-param name="namespace" select="$to_mangle"/>
			</xsl:call-template>			
			<xsl:text>*)&#10;</xsl:text>

		</xsl:for-each>
		<xsl:text>  } > ext_dram&#10;</xsl:text>
		<!-- component bss end -->

		<xsl:text>  __CIAO_</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>DATA_END = . ;&#10;</xsl:text>
		
	</xsl:for-each>
  </xsl:template>


</xsl:stylesheet>
