#!/usr/bin/perl

# Generates messageport_objects.cpp and messageport_objects.h from config.xml
#
# CAVEAT: the make system does not know that this script generates messageport_objects.h,
#   so it is not removed on make clean

use warnings;
use strict;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.cpp$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <network_objects.cpp>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
my $headerfile = $ARGV[2];
$headerfile =~ s/\.cpp$/\.h/;
open (SRC, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";
open (HEADER, ">$headerfile") || die "cannot open $headerfile\n";

print HEADER "#ifndef NETWORKPORT_OBJ_H\n";
print HEADER "#define NETWORKPORT_OBJ_H\n\n";
print HEADER "#include \"os/net/udp.h\"\n";
print HEADER "#include \"os/net/message.h\"\n";
print HEADER "#include \"objects.h\"\n";
print HEADER "#include \"messageport_objects.h\"\n";
print HEADER "#include \"as/AS.h\"\n";
print HEADER "#include \"string.h\"\n";

print HEADER <<eofjs;

#define __byte_swap2(val) \\
    ((((val) & 0xff) << 8) | \\
    (((val) & 0xff00) >> 8))

#define __byte_swap4(val) \\
    ((((val) & 0xff) << 24) | \\
    (((val) & 0xff00) << 8) | \\
    (((val) & 0xff0000) >> 8) | \\
    (((val) & 0xff000000) >> 24))

#define htons(x) __byte_swap2(x)
#define htonl(x) __byte_swap4(x)
#define ntohs(x) __byte_swap2(x)
#define ntohl(x) __byte_swap4(x)

eofjs

print SRC "#include \"$headerfile\"\n\n";

# Generate processPacket
print SRC <<eofjs;
int os::net::Udp::Process(os::net::Message& msg) {
	switch ( msg.udp.header.uh_dport ) {
eofjs
foreach my $nwp ( $cfg->getNetworkPorts() ) {
	my $messagePortSenderSymbol = $nwp->senderComponent()->senderPortSymbol();
	my $port = $nwp->port();
	my $task = $nwp->task();

	print SRC <<eofjs;
		case htons($port): {
			os::net::Message* msgP = $messagePortSenderSymbol.allocateMessage();
			if ( msgP == 0x0 ) {
				return 0;
			}

			memcpy(msgP, &msg, sizeof(msg));
			$messagePortSenderSymbol.sendMessage(msgP);
			AS::SetEvent($task,PacketWaitingEvent);
			return 1;
		}
eofjs
}

print SRC <<eofjs;
		default:
			return -1;
	}
}
eofjs

print HEADER "\n#endif\n";

close(SRC);
close(HEADER);

