#!/usr/bin/perl

# Generates sharedmemory_objects.cpp and sharedmemory_objects.h from config.xml
#
# CAVEAT: the make system does not know that this script generates sharedmemory_objects.h,
#   so it is not removed on make clean

use warnings;
use strict;

use SystemConfiguration;

if(@ARGV != 3 || $ARGV[0] !~ /\.xml$/ || $ARGV[2] !~ /\.cpp$/) {
	print STDERR "Usage: $0 <config.xml> <os.config> <sharedmemory_objects.cpp>\n";
	exit 1;
}

my $cfg = SystemConfiguration->new($ARGV[0], $ARGV[1]);

## Here comes the actual generation code
my $headerfile = $ARGV[2];
$headerfile =~ s/\.cpp$/\.h/;
open (SRC, ">$ARGV[2]") || die "cannot open $ARGV[2]\n";
open (HEADER, ">$headerfile") || die "cannot open $headerfile\n";

print HEADER "#ifndef SHAREDMEMORY_OBJ_H\n";
print HEADER "#define SHAREDMEMORY_OBJ_H\n\n";

print SRC "#include \"$headerfile\"\n\n";

foreach my $shm ($cfg->getSharedMemories()) {
	my $shmk = $shm->{name};

	if ( exists $shm->{cheaderfile} && $shm->{cheaderfile} ne '' ) {
		print HEADER "#include \"$shm->{cheaderfile}\"\n\n";
	}
	if ( exists $shm->{cheaderfile_cpp} && $shm->{cheaderfile_cpp} ne '' ) {
		print SRC "#include \"$shm->{cheaderfile_cpp}\"\n\n";
	}

	my $shmsym = $shm->shmSymbol();
	my $ctype = $shm->areaType();

	print HEADER "#define ${shmk}_SharedMemory $shmsym\n";
	print HEADER "extern $ctype $shmsym;\n";
	print SRC "$ctype $shmsym;\n";
}

print HEADER "\n#endif\n";

close(SRC);
close(HEADER);

